



*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------
* This GAMS file is part of the Electricity Market Model EMMA, written by Lion Hirth, neon Neue Energiekonomik GmbH.
* The model is published under the Creative Commons 3.0 BY-SA License (http://creativecommons.org/licenses/by-sa/3.0/).
* If you do not understand the license agreement, don't use the model.
* Please refer to https://www.pik-potsdam.de/members/hirth/emma and http://neon-energie.de/emma/.
* Feedback, remarks, bug reportings, and suggestions are highly welcome: hirth@neon-energie.de
*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------







*================================================================================================================================================
* 1 SETTINGS and COMMAND LINE PARAMETER DEFAULTS
*================================================================================================================================================



$eolcom //
$phantom emptyset
$setenv GdxCompress 1


$if not set SDIM                 $set SDIM       bench
$if not set YDIM                 $set YDIM       bench
$if not set PURPOSE              $set PURPOSE    test
$if not set HOURS                $set HOURS      100
$if not set OPTFILE              $set OPTFILE    1
$if not set HORIZON              $set HORIZON    long
$if not set SERIES               $set SERIES     1

$if not dexist scen              $call "mkdir scen"






*================================================================================================================================================
* 2 HORIZON INCLUDE FILES
*================================================================================================================================================


$onecho                  > scen\h_mid.gms
capa0(alltec,r)          = i_capa0(alltec,r);
capa0(tec_res,r)         = little;
capaCHP0(tec_chp,r)      = i_capaCHP0(tec_chp,r);
capaCHP0tot(r)           = sum(tec_chp,capaCHP0(tec_chp,r));
DECO.UP(tec_nre,r)       = capa0(tec_nre,r);
$offecho

$onecho                  > scen\h_long.gms
capa0(alltec,r)          = little;
ntc(r,rr)                = little;
capaCHP0(tec_chp,r)      = i_capaCHP0(tec_chp,r);
capaCHP0tot(r)           = sum(tec_chp,capaCHP0(tec_chp,r));
capaCHP0(tec_chp,r)      = 0;
DECO.UP(tec_nre,r)       = capa0(tec_nre,r);
$offecho

$onecho                  > scen\h_short.gms
capa0(alltec,r)          = i_capa0(alltec,r);
capa0(tec_res,r)         = little;
capaCHP0(tec_chp,r)      = i_capaCHP0(tec_chp,r);
capaCHP0tot(r)           = sum(tec_chp,capaCHP0(tec_chp,r));
DECO.UP(tec_nre,r)       = capa0(tec_nre,r);
INVE.UP(tec_nre,r)       = 0;
INVE.UP("shed",r)        = inf;
$offecho

$echo                    > scen\h_none.gms




*================================================================================================================================================
* 3 Y-DIMENSION (YDIM) INCLUDE FILES
*================================================================================================================================================


$onecho  > scen\y_bench.gms
$offecho

$onecho  > scen\y_windshare.gms
INVE.FX("wind",r)                = y.val / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("wind",r)$(y.val=1)      = little;                                        // can also be set to exactly zero
INVE.FX("sola",r)                = little;
$offecho

$onecho  > scen\y_solarshare.gms
INVE.FX("sola",r)                = y.val / 2 / 100 * sum(t,loa(t,r)) / sum(t,profile(t,"sola",r));
INVE.FX("sola",r)$(y.val=1)      = little;
INVE.FX("wind",r)                = little;
$offecho

$onecho  > scen\y_CO2.gms
co2                              = y.val * 6;
co2$(y.val=1)                    = 0;
cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th;
$offecho

$onecho  > scen\y_REScosts.gms
cost_inv(tec_res)                = i_costs(tec_res,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_qfix(tec_res)               = (i_costs(tec_res,"qfixcost") + i_costs(tec_res,"rt_premium")* (8+2)) / sc;
cost_inv("wind")$(ord(y)>1)      = cost_inv("wind")  * (1 - y.val / 100);
cost_qfix("wind")$(ord(y)>1)     = cost_qfix("wind") * (1 - y.val / 100);
cost_inv("sola")$(ord(y)>1)      = cost_inv("sola")  * (1 - y.val / 100 * 2);
cost_qfix("sola")$(ord(y)>1)     = cost_qfix("sola") * (1 - y.val / 100 * 2);
cost_fix(tec_res)                = cost_qfix(tec_res) + cost_inv(tec_res);
cost_fix(tec_res)                = cost_fix(tec_res) + cost_bal(tec_res) * sum(t,profile(t,tec_res,"GER")) / th;
$offecho









*================================================================================================================================================
* 4 S-DIMENSION (SDIM) INCLUDE FILES
*================================================================================================================================================




*===================================
* Series 1 scenario files
*===================================


* Benchmark


$echo    set s /bench/;                                                  > scen\snames_bench.gms
$echo                                                                    > scen\s_bench.gms




* Solar

$echo set s      /"wind (bench)",solar,both,mix/;                        > scen\snames_solar.gms
$onecho                                                                  > scen\s_solar.gms
INVE.FX("wind",r)$(ord(s)=2)     = little;
INVE.FX("sola",r)$(ord(s)>1)     = y.val / 2 / 100 * sum(t,loa(t,r)) / sum(t,profile(t,"sola",r));
INVE.FX("wind",r)$(ord(s)=4)     = 2/3 * y.val / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("sola",r)$(ord(s)=4)     = 1/3 * y.val / 100 * sum(t,loa(t,r)) / sum(t,profile(t,"sola",r));
INVE.FX("wind",r)$(y.val=1)      = little;
INVE.FX("sola",r)$(y.val=1)      = little;
$offecho





* Efficiency

$echo set s      /"low (bench)","high efficiency","eff gas"/;            > scen\snames_efficiency.gms
$onecho                                                                  > scen\s_efficiency.gms
cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th;
cost_var(tec)$(ord(s)=2)         = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff_new") - i_costs(tec,"rt_premium")) / th;
cost_var("CCGT")$(ord(s)=3)      = (i_costs("CCGT","varcost") + (cost_fuel("CCGT") + i_costs("CCGT","co2int")*co2) / i_costs("CCGT","eff_new") - i_costs("CCGT","rt_premium")) / th;
cost_var("OCGT")$(ord(s)=3)      = (i_costs("OCGT","varcost") + (cost_fuel("CCGT") * 1.5 + i_costs("OCGT","co2int")*co2) / i_costs("OCGT","eff_new") - i_costs("OCGT","rt_premium")) / th;
$offecho





* CO2

$echo set s      /"20 /t (bench)","0 /t","50 /t","100 /t"/;          > scen\snames_CO2.gms
$onecho                                                                  > scen\s_CO2.gms
co2$(ord(s)=1)   = 20;
co2$(ord(s)=2)   = 0;
co2$(ord(s)=3)   = 50;
co2$(ord(s)=4)   = 100;
cost_var(tec)    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th;
$offecho




* NoNuc

$echo set s      /"allTech (bench)",allTech100,NoNuc100,NoNucCCS100/;         > scen\snames_NoNuc.gms
$onecho                                                                  > scen\s_NoNuc.gms
INVE.UP("nucl",r)$(ord(s)>2)     = 0;
INVE.UP("lCCS",r)$(ord(s)=4)     = 0;
co2$(ord(s)>1)                   = 100;
cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th; // scenarios that change CO2 or fuel prices need to update variable costs
$offecho





* Fuel

$echo set s      /"todayFuel (bench)",doubleCoal,doubleGas,shaleGas/;    > scen\snames_fuel.gms
*doubleBoth
$onecho                                                                  > scen\s_fuel.gms
cost_fuel(tec)                           = i_fuel(tec,"long-run");
cost_fuel("coal")$(ord(s)=2)             = cost_fuel("coal") * 2;
cost_fuel("CCGT")$(ord(s)=3)             = cost_fuel("CCGT") * 2;
cost_fuel("OCGT")$(ord(s)=3)             = cost_fuel("OCGT") * 2;
cost_fuel("CCGT")$(ord(s)=4)             = cost_fuel("CCGT") / 2;
cost_fuel("OCGT")$(ord(s)=4)             = cost_fuel("OCGT") / 2;
cost_var(tec)                            = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th; // scenarios that change CO2 or fuel prices need to update variable costs
$offecho





* FuelElasticity

$echo set s      /"todayFuel (bench)",coalprice,gasprice,fossilprice/;   > scen\snames_fuelelasticity.gms
$onecho                                                                  > scen\s_fuelelasticity.gms
cost_fuel(tec)                           = i_fuel(tec,"long-run");
cost_fuel("coal")$(ord(s)=2 or ord(s)=4) = cost_fuel("coal") * 1.01;
cost_fuel("CCGT")$(ord(s)=3 or ord(s)=4) = cost_fuel("CCGT") * 1.01;
cost_fuel("OCGT")$(ord(s)=3 or ord(s)=4) = cost_fuel("OCGT") * 1.01;
cost_var(tec)                            = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th; // scenarios that change CO2 or fuel prices need to update variable costs
$offecho





* Invest

$echo set s      /"today (bench)",expensive/;                            > scen\snames_invest.gms
$onecho                                                                  > scen\s_invest.gms
cost_inv(tec_thm)$(ord(s)=2)             = 1.5 * i_costs(tec_thm,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_fix(tec_thm)$(ord(s)=2)             = cost_qfix(tec_thm) + cost_inv(tec_thm);
cost_fix("OCGT")$(ord(s)=2)              = cost_fix("OCGT") * i_costs("OCGT","flex_premium");                                // revenues from ancillary services and/or capacity markets
$offecho





* CostElasticity

$echo set s      /"bench costs (bench)",c_nucl,c_lign,c_coal,c_CCGT,c_OCGT/;     > scen\snames_costelasticity.gms
$onecho                                                                          > scen\s_costelasticity.gms
cost_fuel(tec_thm)               = i_fuel(tec_thm,"long-run");
cost_qfix(tec_thm)               = (i_costs(tec_thm,"qfixcost") + i_costs(tec_thm,"rt_premium")* (8+2)) / sc;              // 8 to allocate RT premium to FC, 2 for actual occured start-ups - but what if a plant runs less than 8000 hours!?!?!? - THIS IS A BUG
cost_inv(tec_thm)                = i_costs(tec_thm,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_var(tec_thm)                = (i_costs(tec_thm,"varcost") + (cost_fuel(tec_thm) + i_costs(tec_thm,"co2int")*co2) / i_costs(tec_thm,"eff") - i_costs(tec_thm,"rt_premium")) / th;

cost_qfix("nucl")$(ord(s)=2)     = 1.01 * cost_qfix("nucl");
cost_inv("nucl")$(ord(s)=2)      = 1.01 * cost_inv("nucl");
cost_var("nucl")$(ord(s)=2)      = 1.01 * cost_var("nucl");

cost_qfix("lign")$(ord(s)=3)     = 1.01 * cost_qfix("lign");
cost_inv("lign")$(ord(s)=3)      = 1.01 * cost_inv("lign");
cost_var("lign")$(ord(s)=3)      = 1.01 * cost_var("lign");

cost_qfix("coal")$(ord(s)=4)     = 1.01 * cost_qfix("coal");
cost_inv("coal")$(ord(s)=4)      = 1.01 * cost_inv("coal");
cost_var("coal")$(ord(s)=4)      = 1.01 * cost_var("coal");

cost_qfix("CCGT")$(ord(s)=5)     = 1.01 * cost_qfix("CCGT");
cost_inv("CCGT")$(ord(s)=5)      = 1.01 * cost_inv("CCGT");
cost_var("CCGT")$(ord(s)=5)      = 1.01 * cost_var("CCGT");

cost_qfix("OCGT")$(ord(s)=6)     = 1.01 * cost_qfix("OCGT");
cost_inv("OCGT")$(ord(s)=6)      = 1.01 * cost_inv("OCGT");
cost_var("OCGT")$(ord(s)=6)      = 1.01 * cost_var("OCGT");

cost_fix(tec_thm)                = cost_qfix(tec_thm) + cost_inv(tec_thm);
cost_fix("OCGT")                 = cost_fix("OCGT") * i_costs("OCGT","flex_premium");
cost_qfix("OCGT")                = cost_qfix("OCGT") * i_costs("OCGT","flex_premium");
$offecho







* NTC

$echo set s      /"endo (bench)",cheapIC,today,zeroNTC,doubleNTC/;       > scen\snames_NTC.gms
$onecho                                                                  > scen\s_NTC.gms
ntc(r,rr)                                = i_NTC(r,rr) / th;
cost_NTC$(ord(s)=2)                      = cost_NTC / 2;
ntc(r,rr)$(ord(s)=3)                     = i_NTC(r,rr) / th;
ntc(r,rr)$(ord(s)=4)                     = 0;
ntc(r,rr)$(ord(s)=5)                     = i_NTC(r,rr) / th * 2;
NTCinv.UP(r,rr)                          = inf;
NTCinv.UP(r,rr)$(ord(s)>2)               = 0;
$offecho




* Storage

$echo set s      /"todaySto (bench)",zeroSto,doubleSto,cheapSto/;        > scen\snames_storage.gms
$onecho                                                                  > scen\s_storage.gms
INVE.UP("pump",r)                        = inf;
DECO.UP("pump",r)                        = capa0("pump",r);
capa0("pump",r)                          = i_capa0("pump",r);
capa0("pump",r)$(ord(s)=2)               = 0;
capa0("pump",r)$(ord(s)=3)               = i_capa0("pump",r) * 2;
INVE.UP("pump",r)$(ord(s)=2 or ord(s)=3) = 0;
DECO.UP("pump",r)$(ord(s)=2 or ord(s)=3) = 0;
cost_fix("pump")$(ord(s)=4 and ord(y)=1) = cost_fix("pump") / 2;
$offecho





* Flex

$echo set s      /"allConstraints (bench)",noAS,noCHP,noAll/;            > scen\snames_flex.gms
$onecho                                                                  > scen\s_flex.gms
as(r)                                    = 0.1 * smax(t,loa(t,r));
as(r)$(ord(s)=2 or ord(s)=4)             = 0;
profile(t,"CHP",r)                       = i_CHP(t,"average");
profile(t,"CHP",r)                       = profile(t,"CHP",r) * (i_CHPyr(r,"CHPyr") * th) / (sum(allt,i_CHP(allt,"average")) * sum(tec_chp,i_capaCHP0(tec_chp,r)*avail(t,tec_chp,r)));
profile(t,"CHP",r)$(ord(s)=3 or ord(s)=4)= 0;
$offecho





* Variability

$echo set s      /"today (bench)",constant,noBalancing,noGrid/;          > scen\snames_variability.gms
$onecho                                                                  > scen\s_variability.gms
loop(year,
profile(t,"wind",r)              = i_wind(t,r,year);
);
test(r)                          = sum(t,profile(t,"wind",r)) * sc;
profile(t,"wind",r)              = profile(t,"wind",r) * i_FLH(r,"FLH") / test(r);
profile(t,"wind",r)$(ord(s)=2)   = i_FLH(r,"FLH")/8760;

as_res                           = 0.05;
as_res$(ord(s)=3)                = 0;
cost_bal(tec_res)                = 4;
cost_bal(tec_res)$(ord(s)=3)     = 0;
$include scen\y_%YDIM%.gms
*** The include file is necessary because YDIM is included before SDIM and hence overwrite chances

ntc(r,rr)                        = i_NTC(r,rr) / th;
ntc(r,rr)$(ord(s)=4)             = i_NTC(r,rr) / th * 1000;
$offecho





* PriceCap

$echo set s      /"1000 (bench)","500","250","150"/;                 > scen\snames_PriceCap.gms
$onecho                                                                  > scen\s_PriceCap.gms
cost_var("shed")$(ord(s)=2)      = 500 / th;
cost_var("shed")$(ord(s)=3)      = 250 / th;
cost_var("shed")$(ord(s)=4)      = 120 / th;
$offecho




* Wind Profiles

$echo set s      /"2010p (bench)",2008p,2009p,BASEp,GERp,offshore/;      > scen\snames_WindProfiles.gms
$onecho                                                                  > scen\s_WindProfiles.gms
year("2008")                     = no;
year("2009")                     = no;
year("2010")                     = no;
year("2010")$(ord(s)=1)          = yes;
year("2008")$(ord(s)=2)          = yes;
year("2009")$(ord(s)=3)          = yes;
loop(year,
profile(t,"wind",r)              = i_wind(t,r,year);
profile(t,"sola",r)              = i_sola(t,r,year);
);
profile(t,"wind",r)$(ord(s)=4)   = i_FLH(r,"FLH")/8760;
profile(t,"wind",r)$(ord(s)=5)   = i_wind(t,"GER","2010");
profile(t,"wind",r)$(ord(s)=6)   = i_offs(t,r,"2010");
test(r)                          = sum(t,profile(t,"wind",r)) * sc;
profile(t,"wind",r)              = profile(t,"wind",r) * i_FLH(r,"FLH") / test(r);
$offecho




* Wind Profiles 2

$echo set s      /"2010on (bench)",2010off,BASEp/;                       > scen\snames_WindProfiles2.gms
$onecho                                                                  > scen\s_WindProfiles2.gms
profile(t,"wind",r)              = i_wind(t,r,"2010");
profile(t,"wind",r)$(ord(s)=2)   = i_offs(t,r,"2010");
profile(t,"wind",r)$(ord(s)=3)   = 2000/8760;
test(r)                          = sum(t,profile(t,"wind",r)) * sc;
profile(t,"wind",r)              = profile(t,"wind",r) * 2000 / test(r);
$offecho




* Forerunner Germany

$echo set s      /all,Germany/;                                          > scen\snames_Forerunner.gms
$onecho                                                                  > scen\s_Forerunner.gms
INVE.FX("wind",r)$(ord(s)=2)     = little;
INVE.FX("wind","GER")$(ord(s)=2) = y.val / 100 * sum((t,r),loa(t,r)) * sc / 2000;
$offecho




* Horizon

$echo set s      /"medium (bench)",long,half-long,short/;                > scen\snames_horizon.gms
$onecho                                                                  > scen\s_horizon.gms
INVE.UP(tec_nre,r)               = inf;
capa0(alltec,r)                  = i_capa0(alltec,r);
capa0(tec_res,r)                 = little;
capaCHP0(tec_chp,r)              = i_capaCHP0(tec_chp,r);
capaCHP0tot(r)                   = sum(tec_chp,capaCHP0(tec_chp,r));
ntc(r,rr)                        = i_NTC(r,rr) / th;
NTCinv.UP(r,rr)                  = inf;

capa0(alltec,r)$(ord(s)=2)       = little;
capaCHP0(tec_chp,r)$(ord(s)=2)   = 0;
ntc(r,rr)$(ord(s)=2)             = little;

capa0(alltec,r)$(ord(s)=3)       = i_capa0(alltec,r) / 2;
capa0(tec_res,r)$(ord(s)=3)      = little;
capaCHP0(tec_chp,r)$(ord(s)=3)   = capaCHP0(tec_chp,r) / 2;
ntc(r,rr)$(ord(s)=2)             = i_NTC(r,rr) / th / 2;

INVE.UP(tec_nre,r)$(ord(s)=4)    = 0;
INVE.UP("shed",r)$(ord(s)=4)     = inf;
NTCinv.UP(r,rr)$(ord(s)=4)       = 0;

DECO.UP(alltec,r)                = capa0(alltec,r);

$offecho





*===================================
* Series 2 scenario files
*===================================




* Capa Backtest

$echo set s      /long/;                                                 > scen\snames_backtest.gms
$onecho                                                                  > scen\s_backtest.gms
co2                              = 0;
cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th; // scenarios that change CO2 or fuel prices need to update variable costs
avail(t,alltec,r)                = 0.8;
INVE.UP(alltec,r)                = inf;
INVE.UP("lign",r)$(not r_lign(r))= 0;
INVE.LO(alltec,r)                = 0;
INVE.UP("lign","GER")$(ord(y)>1) = 25;
INVE.UP("lign","POL")$(ord(y)>1) = 15;
capa0(alltec,r)                  = i_capa0(alltec,r);
capa0(tec_res,r)                 = little;
capa0(alltec,r)                  = little;
capaCHP0(tec_chp,r)              = i_capaCHP0(tec_chp,r);
capaCHP0tot(r)                   = sum(tec_chp,capaCHP0(tec_chp,r));
capaCHP0(tec_chp,r)              = 0;
DECO.UP(alltec,r)                = capa0(alltec,r);
DECO.FX(tec_res,r)               = 0;
$offecho




* Small Steps

$echo set s      /med1*med5/;                                            > scen\snames_smallSteps.gms
$onecho                                                                  > scen\s_smallSteps.gms
INVE.FX("wind",r)                = y.val/4     / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("wind",r)$(y.val=1)      = 0;
INVE.FX("wind",r)$(ord(s)=2)     =(y.val/4+9)  / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("wind",r)$(ord(s)=3)     =(y.val/4+18) / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("wind",r)$(ord(s)=4)     =(y.val/4+27) / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("wind",r)$(ord(s)=5)     =(y.val/4+36) / 100 * sum(t,loa(t,r)) * sc / 2000;
$offecho




* High shares (first run is not the benchmark)

$echo set s      /wind,solar,both/;                                      > scen\snames_highShares.gms
$onecho                                                                  > scen\s_highShares.gms
INVE.FX("wind",r)                        = y.val / 100 * sum(t,loa(t,r)) * sc / 2000 * 2.5;
INVE.FX("wind",r)$(y.val=1)              = little;
INVE.FX("sola",r)                        = y.val / 2 / 100 * sum(t,loa(t,r)) / sum(t,profile(t,"sola",r)) * 2.5;
INVE.FX("sola",r)$(y.val=1)              = little;
INVE.LO("wind",r)$(ord(s)=2)             = 0;
INVE.LO("sola",r)$(ord(s)=1)             = 0;
$offecho




* High reductions

$echo set s      /wind,solar,both/;                                      > scen\snames_highReductions.gms
$onecho                                                                  > scen\s_highReductions.gms
cost_inv(tec_res)                = i_costs(tec_res,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_qfix(tec_res)               = (i_costs(tec_res,"qfixcost") + i_costs(tec_res,"rt_premium")* (8+2)) / sc;
cost_inv("wind")$(ord(y)>1 and (ord(s)=1 or ord(s)=3))      = cost_inv("wind")  * (1 - y.val / 100 * 2);
cost_qfix("wind")$(ord(y)>1 and (ord(s)=1 or ord(s)=3))     = cost_qfix("wind") * (1 - y.val / 100 * 2);
cost_inv("sola")$(ord(y)>1 and (ord(s)=2 or ord(s)=3))      = cost_inv("sola")  * (1 - y.val / 100 * 3);
cost_qfix("sola")$(ord(y)>1 and (ord(s)=2 or ord(s)=3))     = cost_qfix("sola") * (1 - y.val / 100 * 3);
cost_fix(tec_res)                = cost_qfix(tec_res) + cost_inv(tec_res);
$offecho




* CheapRES

$echo    set s /wind,solar,both,noNuc/;                                  > scen\snames_cheapRES.gms
$onecho                                                                  > scen\s_cheapRES.gms
cost_inv(tec_res)                = i_costs(tec_res,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_qfix(tec_res)               = (i_costs(tec_res,"qfixcost") + i_costs(tec_res,"rt_premium")* (8+2)) / sc;
cost_inv("wind")$(ord(s)<>2)     = cost_inv("wind")  * 0.7;
cost_qfix("wind")$(ord(s)<>2)    = cost_qfix("wind") * 0.7;
cost_inv("sola")$(ord(s)>1)      = cost_inv("sola")  * 0.4;
cost_qfix("sola")$(ord(s)>1)     = cost_qfix("sola") * 0.4;
cost_fix(tec_res)                = cost_qfix(tec_res) + cost_inv(tec_res);
cost_fix(tec_res)                = cost_fix(tec_res) + cost_bal(tec_res) * sum(t,profile(t,tec_res,"GER")) / th;
INVE.UP("nucl",r)$(ord(s)=4)     = 0;
$offecho


* CheapRES2

$echo    set s /cheapRES/;                                               > scen\snames_cheapRES2.gms
$onecho                                                                  > scen\s_cheapRES2.gms
cost_inv(tec_res)                = i_costs(tec_res,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_qfix(tec_res)               = (i_costs(tec_res,"qfixcost") + i_costs(tec_res,"rt_premium")* (8+2)) / sc;
cost_inv("wind")                 = cost_inv("wind")  * 0.7;
cost_qfix("wind")                = cost_qfix("wind") * 0.7;
cost_inv("sola")                 = cost_inv("sola")  * 0.4;
cost_qfix("sola")                = cost_qfix("sola") * 0.4;
cost_fix(tec_res)                = cost_qfix(tec_res) + cost_inv(tec_res);
cost_fix(tec_res)                = cost_fix(tec_res) + cost_bal(tec_res) * sum(t,profile(t,tec_res,"GER")) / th;
$offecho





* HighRES

$echo    set s /wind,solar,both/;                                        > scen\snames_HighRES.gms
$onecho                                                                  > scen\s_HighRES.gms
cost_inv(tec_res)                = i_costs(tec_res,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_qfix(tec_res)               = (i_costs(tec_res,"qfixcost") + i_costs(tec_res,"rt_premium")* (8+2)) / sc;

INVE.FX("wind",r)                = little;
INVE.FX("sola",r)                = little;

INVE.FX("wind",r)$(ord(s)<>2)    = 30 / 100 * sum(t,loa(t,r)) * sc / 2000;
INVE.FX("sola",r)$(ord(s)<>1)    = 15 / 100 * sum(t,loa(t,r)) * sc / 2000;


$offecho





* Phase-Out

$echo set s      /"medium term (bench)","100 /t",Phase-Out,Phase-Out100/;       > scen\snames_Phase-Out.gms
$onecho                                                                          > scen\s_Phase-Out.gms
INVE.UP("nucl","GER")$(ord(s)>2) = 0;
capa0("nucl","GER")$(ord(s)>2)   = 0;
co2                              = 20;
co2$(ord(s)=2 or ord(s)=4)       = 100;
cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th; // scenarios that change CO2 or fuel prices need to update variable costs
$offecho




* 50VRE

$echo set s      /"all techs",noNuc,noNucCCS/;                                   > scen\snames_50VRE.gms
$onecho                                                                          > scen\s_50VRE.gms
INVE.FX("wind",r)                = 0.4 * sum(t,loa(t,r)) / sum(t,profile(t,"wind",r));
INVE.FX("sola",r)                = 0.1 * sum(t,loa(t,r)) / sum(t,profile(t,"sola",r));
INVE.UP("nucl",r)$(ord(s)>1)     = 0;
INVE.UP("lCCS",r)$(ord(s)=3)     = 0;
$offecho









*===================================
* Series 3 scenario files
*===================================


* Redistribution


$echo set s      /"P1","P2-wind","P2-CO2","P2-both"/;                    > scen\snames_redistribution.gms
$onecho                                                                  > scen\s_redistribution.gms


* Both periods
avail(t,alltec,r)                = 0.8;



* First Period (Long-term)

INVE.UP(alltec,r)                = inf;
INVE.UP("lign",r)$(not r_lign(r))= 0;
INVE.LO(alltec,r)                = 0;
NTCinv.UP(r,rr)                  = inf;
capaCHP0(tec_chp,r)              = i_capaCHP0(tec_chp,r);
capaCHP0tot(r)                   = sum(tec_chp,capaCHP0(tec_chp,r));

if(ord(s)=1,
capa0(alltec,r)                  = 0;
ntc(r,rr)                        = 0;
ntc(rr,r)                        = 0;
capaCHP0(tec_chp,r)              = 0;
INVE.FX("wind",r)                = 0;
co2                              = 0;

lt_INVE(alltec,r)$(ord(y)=5)     = INVE.L(alltec,r);                             // can be written before solve statement because of looping
lt_NTCinv(r,rr)$(ord(y)=5)       = NTCinv.L(r,rr);
CAPACHP.L(tec_chp,r)$(ord(y)=1)  = 1;                                            // ich verstehe immer noch nicht warum das hier notwendig ist (ist es aber)
lt_CAPACHP(tec_chp,r)$(ord(y)=5) = CAPACHP.L(tec_chp,r);
);


* Second Period (Medium-term)

if(ord(s)>1,
capa0(alltec,r)                  = lt_INVE(alltec,r) + little;           // without adding capacities: scarcity prices
*capa0("OCGT",r)                  = capa0("OCGT",r) + little;
ntc(r,rr)                        = lt_NTCinv(r,rr);                      // + little;
ntc(r,rr)                        = lt_NTCinv(r,rr);                      // + little;
capaCHP0(tec_chp,r)              = lt_CAPACHP(tec_chp,r);                // + little;
INVE.UP("nucl",r)                = 0;
);

DECO.UP(alltec,r)                = capa0(alltec,r);                      // braucht man das?
DECO.FX(tec_res,r)               = 0;                                    // braucht man das?



* Second Period: y-dimension

if((ord(s)=2) or (ord(s)=4),
INVE.FX("sola",r)                = y.val / 2 / 100 * sum(t,loa(t,r)) / sum(t,profile(t,"sola",r));
INVE.FX("sola",r)$(y.val=1)      = 0;
*INVE.FX("wind",r)                = y.val / 100 * sum(t,loa(t,r)) * sc / 2000;
*INVE.FX("wind",r)$(y.val=1)      = 0;
);

if(ord(s)>2,
co2                              = y.val * 3.3;
co2$(y.val=1)                    = 0;
INVE.FX("wind",r)$(ord(y)=1)     = 0;                                            // wie in P1
);


* Both Periods

INVE.UP("lign","GER")            = 25 - capa0("lign","GER") + little;
INVE.UP("lign","POL")            = 15 - capa0("lign","POL") + little;
INVE.UP("lign","GER")$(INVE.UP("lign","GER")<0)            = 0;                  // otherwise, the upper limit seems to be still negative -> 2 model errors
INVE.UP("lign","POL")$(INVE.UP("lign","POL")<0)            = 0;

cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th; // scenarios that change CO2 or fuel prices need to update variable costs
cost_var("shed")                 = cost_var("shed") * 1000;


display capa0, cost_fix, cost_qfix, cost_var, capaCHP0, capaCHP0tot, CAPACHP.L, INVE.L;

$offecho





*===================================
* Backtesting
*===================================


* DOES NOT WORK AT THE MOMENT
* Calibration (2008,2009,2010) [don't use second dimension s]
$ONTEXT
capa0(alltec,r)          = i_capa0(alltec,r);
INVE.UP(alltec,r)        = 0;
INVE.UP("shed",r)        = inf;
DECO.UP(alltec,r)        = 0;
NTCinv.UP(r,rr)          = 0;
inveCHP.UP(tec_chp,r)    = 0;
decoCHP.UP(tec_chp,r)    = 0;

year("2009")             = no;
year("2010")             = no;
year("2008")             = yes;
year("2009")$(ord(y)=2)  = yes;
year("2010")$(ord(y)=3)  = yes;

loop(year,
loa(t,r)                 = i_loa(t,r,year) / th;
pri(t,r)                 = i_pri(t,r,year);
profile(t,"wind",r)      = i_wind(t,r,year);
profile(t,"sola",r)      = i_sola(t,r,year);
profile(t,"CHP",r)       = i_CHP(t,"average");
cost_fuel(tec)           = i_fuel(tec,year);
);

capa0("sola",r)                  = 0;
capa0("sola","GER")              = 2;
capa0("sola","GER")$(ord(y)=2)   = 6;
capa0("sola","GER")$(ord(y)=3)   = 10;
capa0("wind","GER")              = 21;
capa0("wind","GER")$(ord(y)=2)   = 23;
capa0("wind","GER")$(ord(y)=3)   = 25;

avail(t,alltec,r)        = 0.8;
avail(t,alltec,"POL")    = 0.6;
avail(t,"nucl","FRA")    = sum(tmonth(t,month),i_monthly(month,"avail_FRA_nuc"));
avail(t,alltec,r)$(ord(y)=1)        = avail(t,alltec,r) * 0.97;
avail(t,alltec,r)$(ord(y)=2)        = avail(t,alltec,r) * 0.98;

loa(t,"FRA")$(loa(t,"FRA")>53)   = (loa(t,"FRA")-53) * 0.5 + 53;
loa(t,"GER")                     = loa(t,"GER") / 0.91;
cost_var(tec)                    = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th;
LOAD.LO(t,r)                     = loa(t,r);
$OFFTEXT









*================================================================================================================================================
* 5 SETS
*================================================================================================================================================


SET

allt             all possible hours      /1*8760/
allr             all possible regions    /GER,FRA,NLD,IRL,GBR,PRT,ESP,BEL,LUX,DKW,DKE,NOR,SWE,FIN,CHE,ITA,AUT,SVN,POL,CZE,SVK,HUN,GRC,ROU,HRV,BIH,SRB,MNE,MKD,BGR/
alltec           all possible techs      /nucl,lCCS,lign,coal,CCGT,OCGT,shed,wind,sola,pump/
allyear          all historical years    /2008*2010,long-run/

t(allt)          hour of the year        /1*%HOURS%/
ti(t)            short for reporting     /1*100/
r(allr)          regions                 /GER,FRA,POL,NLD,BEL/                   // ,GBR,DKW,DKE
r_h(r)           regions with PHS        /GER,FRA/                               // subset of r or "emptyset"
r_lign(r)        regions with lignite    /GER,POL/                               // subset of r or "emptyset"
tec(alltec)      generation technology   /nucl,lCCS,lign,coal,CCGT,OCGT,shed,wind,sola/
tec_res(tec)     must-run techs          /wind,sola/
tec_nre(alltec)  non-res techs           /nucl,lCCS,lign,coal,CCGT,OCGT,shed,pump/
tec_thm(tec)     dispatchable techs      /nucl,lCCS,lign,coal,CCGT,OCGT,shed/
tec_chp(tec)     CHP supporting techs    /lign,coal,CCGT,OCGT,shed/              // "shed" is included to make SHORT runs feasible with new CHP specification; I'm not sure if this is ok
tec_flex(alltec) flexible techs          /OCGT,pump/

peak             peak and off-peak       /p,op/
day              day of the year         /1*365/
hourd            hour of the day         /1*24/
hourw            hour of the week        /1*168/
month            month of the year       /jan,feb,mar,apr,may,jun,jul,aug,sep,oct,nov,dec/
weekday09        weekday in 2009         /mon,tue,wed,thu,fri,sat,sun/
season           season                  /winter,spring,summer,fall/
tpeak(t,peak)    mapping
tday(t,day)      mapping
thourd(t,hourd)  mapping
thourw(t,hourw)  mapping
tmonth(t,month)  mapping
tseason(t,season) mapping
time(allt,peak,day,hourd,hourw,month,season,weekday09)  super-set for time sets

year(allyear)    historical year         /2010/


var              in order of appearance  /"Demand"               // volumes
                                          "Base Price"           // prices (following)
                                          "Peak Price"
                                          "Off-Peak Price"
                                          "Load Price"
                                          "Scarcity Price"
                                          "P-to-OP"
                                          "# zero prices"
                                          "# scarcity prices"
                                          "Total Sys Cost"       // values (following)
                                          "Gen Costs"
                                          "IC cost"
                                          "Gen Costs nRES"
                                          "Gen Costs RES"
                                          "Gen Costs w/o CO2"
                                          "Total Sys Cost w sunk"
                                          "Gen Costs w sunk"
                                          "IC cost w sunk"
                                          "Gen Costs nRES w sunk"
                                          "Gen Costs RES w sunk"
                                          "Spot Expenses"
                                          "Spot Income"
                                          "Spot Conv Income"
                                          "AS Income"
                                          "Heat Income"
                                          "Sunk gen cost"
                                          "Sunk conv gen cost"
                                          "Sunk IC cost"
                                          "Conv Rent"
                                          "Conv rent - sunk cost"
                                          "RES tax"
                                          "CO2 revenue"
                                          "Consumer Rent"
                                          "Producer Rent"
                                          "State Revenue"
                                          "Economic Welfare"/




var_onlysys(var) reporting names that are only possible at system level                                          /"Total Sys Cost","Total Sys Cost w sunk"/
var_o_val(var)   reporting names that are contained in o_val (values that exist at country and system level)
var_val(var)     reporting names that indicate values (including those only possible at system level)
var_price(var)   reporting names that indicate prices (which cannot be summed or written in absolute values)     /"Base Price","Peak Price","Off-Peak Price","Load Price","Scarcity Price","P-to-OP","# zero prices","# scarcity prices"/
var_vol(var)     reporting names that indicate volumes                                                           /"Demand"/

r_tec_var        names in r_tex          /"capa0 (GW)","new cap (GW)","decom cap (GW)","net gen (TWh)","New Fix Cost (G)","Old qfix Cost (G)","Var Cost (G)","Revenue Spot (G)","Revenue AS (G)","Revenue Heat (G)","SR Profit (G)"/

y                scenarios: first dim    /1,5,10,20,30/          // 2008,2009,2010  1,20,40,60,100  1,20,50,100,200  1,5,10,20,30
;


var_o_val(var)           = yes;
var_o_val(var_onlysys)   = no;
var_o_val(var_price)     = no;
var_o_val(var_vol)       = no;
var_val(var_o_val)       = yes;
var_val(var_onlysys)     = yes;

$include scen\snames_%SDIM%.gms


ALIAS (r,rr)






*================================================================================================================================================
* 6 PARAMETER DEFINITIONS
*================================================================================================================================================



PARAMETER

* Input parameters read in from XLS (i_)
i_loa(allt,allr,allyear)
i_pri(allt,allr,allyear)
i_wind(allt,allr,allyear)
i_offs(allt,allr,allyear)
i_sola(allt,allr,allyear)
i_CHP(allt,*)
i_CHPyr(allr,*)
i_monthly(month,*)
i_costs(alltec,*)
i_FLH(allr,*)
i_NTC(allr,allr)
i_km(allr,allr)
i_ACDC(allr,allr)
i_capa0(alltec,allr)
i_capaCHP0(alltec,allr)
i_fuel(tec,allyear)


* Model Parameters
pri(t,r)                 hourly price from EEX                   ( per MWh)
loa(t,r)                 hourly load                             (GW)                    [loaded as MW]
profile(t,*,r)           generation profiles for wind solar CHP  (1 | sum up to FLH)
co2                      CO2-price                               ( per t)
cost_fix(alltec)         annulized fixed costs (qfix + inv)      (M per GW*a)
cost_qfix(alltec)        annulized quasi-fixed costs             (M per GW*a)
cost_inv(alltec)         annualized investment cost              (M per GW*a)
cost_var(tec)            variable cost                           (M per GWh)            [loaded as  per MWh]
cost_fuel(tec)           fuel cost                               ( per MWht)
cost_NTC                 transmission cost                       (M per GW_NTC*km*a)
cost_bal(tec_res)        balancing cost                          ( per MWh)
capa0(alltec,r)          existing net installed total capacity   (GW)
capaCHP0(alltec,r)       existing CHP generation capacity        (GW)
capaCHP0tot(r)           sum of existing CHP generation capacity (GW)
avail(t,alltec,r)        availability of generation              (1)
NTC(r,rr)                trans capacity from r to rr             (GW) [rescaled]
km(r,rr)                 distance between regions                (km)
ACDC(r,rr)               HVAC or HVDC dummy                      (dummy)
as(r)                    must-run                                (GW)
as_res                   must-run as share of VRE capacity       (1)


* Units
* M / GW*a      =  / KW*a
*  / MWh        = M / TWh
*  / KWh        = M / GWh
*  / KW         = M / GW


* Outcome Parameters (o_)
o_capa(*,r)              total installed capacity        (M per full a)
o_ygene(*,r)             yearly generation               (TWh per full a)
o_cur(r)                 curtailed generation            (TWh per full a)
o_ASp(r)                 AS price                        ( per KW full a)
o_share(alltec,*)        market share                    (%)
o_cost(alltec,r)         yearly total cost by tec        (M per full a)
o_revS(alltec,r)         yearly revenues from spot       (M per full a)
o_revA(alltec,r)         yearly revenues from AS         (M per full a)
o_revH(alltec,r)         yearly revenues from Heat-MR    (M per full a)
o_rev(alltec,r)          total yearly revenues           (M per full a)
o_bp(r)                  base price                      ( per MWh)
o_dem(r)                 yearly demand                   (TWh per full a)
o_gen(r)                 yearly generation               (TWh per full a)
o_CO2(r)                 CO2 emissions                   (Mt per full a)
o_val(r,s,y,var)         output: values                  (G per full a)
lt_INVE(alltec,r)        outcome of the LT run
lt_NTCinv(r,rr)          outcome of the LT run
lt_CAPACHP(tec_chp,r)    outcome of the LT run



* Reporting Parameters (r_)
r_s
r_r
r_r2
r_vf
r_vf2
r_capa
r_capa2
r_ms
r_ms2
r_gene
r_tec
r_sto
r_NTC
r_t
r_t2
r_t3
r_t4
r_d
r_set
r_set2
r_mo
r_cal
r_cal2
r_cal3
r_cal4
r_mod


* Scalars, scale, test
th                       /1000/
mn                       /1000000/
little                   /0.00001/
discountrate             /0.07/
lifetime                 /25/
sc                       scale parameter for t<8760
series                   /%SERIES%/
test
test2
test3
;





*================================================================================================================================================
* 7 PARAMETER VALUES
*================================================================================================================================================



*===================================
* a) Read in from Exel
*===================================



* Time Series Data

$onecho > data_ts.txt
set=time         rng=time!a2:k8761       rdim=8
par=i_loa        rng=loa!a6:aq8767       rdim=1 cdim=2
par=i_pri        rng=pri!a6:aq8767       rdim=1 cdim=2
par=i_wind       rng=wind!a6:aq8767      rdim=1 cdim=2
par=i_offs       rng=offs!a6:aq8767      rdim=1 cdim=2
par=i_sola       rng=sola!a6:aq8767      rdim=1 cdim=2
par=i_CHP        rng=CHP!a6:d8800        rdim=1 cdim=1
$offecho

$IF NOT EXIST data_ts.gdx $CALL GDXXRW.exe input\data_ts.xls @data_ts.txt
$GDXIN data_ts.gdx
$LOADdc time, i_loa, i_pri, i_wind, i_offs, i_sola, i_CHP


* Other data

$onecho > data.txt
par=i_costs      rng=cost!a2:j14
par=i_capa0      rng=capa0!a2:p14
par=i_capaCHP0   rng=capa0!a25:p37
par=i_CHPyr      rng=CHPyr!a2:b20
par=i_fuel       rng=fuel!a2:e9
par=i_monthly    rng=monthly!a2:c14
par=i_FLH        rng=FLH!a2:b20
par=i_NTC        rng=NTC!b2:af32
par=i_km         rng=km!b2:af32
par=i_ACDC       rng=ACDC!b2:af32
$offecho

$IF NOT EXIST data.gdx $CALL GDXXRW.exe input\data.xls @data.txt
$GDXIN data.gdx
$LOADdc i_costs, i_capa0, i_capaCHP0, i_CHPyr, i_fuel, i_monthly, i_FLH, i_NTC, i_km, i_ACDC



*===================================
* b) Model parameters from inputs
*===================================


* Time-dependend operations

loop(time(t,peak,day,hourd,hourw,month,season,weekday09),
tpeak(t,peak)    = yes;
tday(t,day)      = yes;
thourd(t,hourd)  = yes;
thourw(t,hourw)  = yes;
tmonth(t,month)  = yes;
tseason(t,season)= yes;
);



* Availability

avail(t,alltec,r)        = 0.8;
avail(t,alltec,"POL")    = 0.6;
avail(t,"nucl","FRA")    = sum(tmonth(t,month),i_monthly(month,"avail_FRA_nuc"));



* Scalars

sc                       = 8760 / card(t);
co2                      = 20;



* Hourly profiles

loop(year,
loa(t,r)                 = i_loa(t,r,year) / th;
pri(t,r)                 = i_pri(t,r,year);
profile(t,"wind",r)      = i_wind(t,r,year);
profile(t,"sola",r)      = i_sola(t,r,year);
profile(t,"CHP",r)       = i_CHP(t,"average");
cost_fuel(tec)           = i_fuel(tec,year);
);
profile(t,"CHP",r)       = profile(t,"CHP",r) * (i_CHPyr(r,"CHPyr") * th) / (sum(allt,i_CHP(allt,"average")) * sum(tec_chp,i_capaCHP0(tec_chp,r)*avail(t,tec_chp,r)));
test(r)                  = sum(t,profile(t,"wind",r)) * sc;
profile(t,"wind",r)      = profile(t,"wind",r) * i_FLH(r,"FLH") / test(r);



* Interconnector-parameters

ntc(r,rr)                = i_NTC(r,rr) / th;
km(r,rr)                 = i_km(r,rr);
ACDC(r,rr)               = i_ACDC(r,rr);




* Ancillary service

as(r)                    = 0.1 * smax(t,loa(t,r));
as_res                   = 0.05;




* Costs

cost_bal(tec_res)        = 4;
cost_fuel(tec)           = i_fuel(tec,"long-run");
cost_qfix(alltec)        = (i_costs(alltec,"qfixcost") + i_costs(alltec,"rt_premium")* (8+2)) / sc;              // 8 to allocate RT premium to FC, 2 for actual occured start-ups - but what if a plant runs less than 8000 hours!?!?!? - THIS IS A BUG
cost_inv(alltec)         = i_costs(alltec,"invest") * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;
cost_fix(alltec)         = cost_qfix(alltec) + cost_inv(alltec);
cost_fix(tec_res)        = cost_fix(tec_res) + cost_bal(tec_res) * sum(t,profile(t,tec_res,"GER")) / th;
cost_fix(tec_flex)       = cost_fix(tec_flex) * i_costs(tec_flex,"flex_premium");                                // revenues from ancillary services and/or capacity markets
cost_qfix(tec_flex)      = cost_qfix(tec_flex) * i_costs(tec_flex,"flex_premium");
cost_var(tec)            = (i_costs(tec,"varcost") + (cost_fuel(tec) + i_costs(tec,"co2int")*co2) / i_costs(tec,"eff") - i_costs(tec,"rt_premium")) / th;
cost_NTC                 = 1 * ((1+discountrate)**lifetime*discountrate)/((1+discountrate)**lifetime-1) / sc;





* Ad-hoc corrections
* Proxy for French hydro (demand reduced by ca. 55 TWh)
loa(t,"FRA")$(loa(t,"FRA")>53)   = (loa(t,"FRA")-53) * 0.5 + 53;
* Scaling up German load (https://www.entsoe.eu/fileadmin/user_upload/_library/publications/ce/Load_and_Consumption_Data.pdf)
loa(t,"GER")                     = loa(t,"GER") / 0.91;






*================================================================================================================================================
* 8 MODEL
*================================================================================================================================================



*===================================
* a) Model Formulation
*===================================


VARIABLES
LOAD(t,r)        load plus curtailment   (GW)
INVE(alltec,r)   investment              (GW)
DECO(alltec,r)   disinvestments          (GW)
GENE(t,alltec,r) generation              (GW)
CAPACHP(tec_chp,r) CHP capacity          (GW)
inveCHP(tec_chp,r)
decoCHP(tec_chp,r)
FLOW(t,r,rr)     exports from r to rr    (GW)
COST             total system cost       (M)
STO_CAP(r)       storage capacity        (GWh)
STO_V(t,r)       stored energy           (GWh)
STO_I(t,r)       in-feed to storage      (GW)
NTCinv(r,rr)     new trans capacity      (GW)


POSITIVE VARIABLES LOAD, INVE, DECO, GENE, CAPACHP, inveCHP, decoCHP, STO_CAP, STO_V, STO_I, NTCinv, ASC;



EQUATION E1, E1a, E2, E2a, E3, E3a, E3b, E3c, E3d, E3e, E4, E4a, E4b, E4c, E5, E5a, E5b, E6, E6a, E6b, E6c, E6d, E6e, E9;


E1(t,r)$(not r_h(r))..   LOAD(t,r)         =E= sum(tec,GENE(t,tec,r)) - sum(rr,FLOW(t,r,rr));                                            // sum(rr,FLOW(t,r,rr)*1.1): this is not a correct representation of losses?
E1a(t,r)$(r_h(r))..      LOAD(t,r)         =E= sum(tec,GENE(t,tec,r)) - sum(rr,FLOW(t,r,rr)) + i_costs("pump","eff") * GENE(t,"pump",r) - STO_I(t,r);

E2(t,tec_res,r)..        GENE(t,tec_res,r) =E= (capa0(tec_res,r) + INVE(tec_res,r)) * profile(t,tec_res,r);
E2a(t,tec_thm,r)..       GENE(t,tec_thm,r) =L= (capa0(tec_thm,r) + INVE(tec_thm,r) - DECO(tec_thm,r)) * avail(t,tec_thm,r);

E3(t,tec_chp,r)..        GENE(t,tec_chp,r) =G= CAPACHP(tec_chp,r) * profile(t,"CHP",r) * avail(t,tec_chp,r);                            // ideally, "avail" should go (but problem could become infeasible)
E3a(r)..                 capaCHP0tot(r)    =E= sum(tec_chp,CAPACHP(tec_chp,r));
E3b(tec_chp,r)..         CAPACHP(tec_chp,r)=E= capaCHP0(tec_chp,r) + inveCHP(tec_chp,r) - decoCHP(tec_chp,r);
E3c(tec_chp,r)..         inveCHP(tec_chp,r)=L= INVE(tec_chp,r);                                                                         // redundant?
E3d(tec_chp,r)..         decoCHP(tec_chp,r)=L= DECO(tec_chp,r);                                                                         // redundant?
E3e(tec_chp,r)..         CAPACHP(tec_chp,r)=L= capa0(tec_chp,r) + INVE(tec_chp,r) - DECO(tec_chp,r);                                    // redundant?

E4(t,r,rr)$km(r,rr)..    FLOW(t,r,rr)      =E= -FLOW(t,rr,r);
E4a(t,r,rr)$km(r,rr)..   FLOW(t,r,rr)      =L= ntc(r,rr) + NTCinv(r,rr);
E4b(t,rr,r)$km(rr,r)..   FLOW(t,rr,r)      =L= ntc(rr,r) + NTCinv(rr,r);
E4c(r,rr)$km(r,rr)..     NTCinv(r,rr)      =E= NTCinv(rr,r);

E5(r)..                  ASC(r)            =G= as(r) + as_res * sum(tec_res,(capa0(tec_res,r) + INVE(tec_res,r)));
E5a(t,r)$(not r_h(r))..  ASC(r)            =L= sum(tec_thm,GENE(t,tec_thm,r))- sum(tec_chp,CAPACHP(tec_chp,r) * profile(t,"CHP",r) * avail(t,tec_chp,r));
E5b(t,r)$(r_h(r))..      ASC(r)            =L= sum(tec_thm,GENE(t,tec_thm,r))- sum(tec_chp,CAPACHP(tec_chp,r) * profile(t,"CHP",r) * avail(t,tec_chp,r)) + i_costs("pump","eff") * GENE(t,"pump",r) + STO_I(t,r);

E6(t,r)$(r_h(r))..       STO_V(t,r)        =E= STO_V(t-1,r) + STO_I(t,r) - GENE(t,"pump",r);
E6a(t,r)$(r_h(r))..      STO_I(t,r)        =L= capa0("pump",r) + INVE("pump",r) - DECO("pump",r);
E6b(t,r)$(r_h(r))..      GENE(t,"pump",r)  =L= capa0("pump",r) + INVE("pump",r) - DECO("pump",r);
E6c(t,r)$(r_h(r))..      STO_V(t,r)        =L= 8 * (INVE("pump",r) + capa0("pump",r));
E6d(t,r)$(not r_h(r))..  GENE(t,"pump",r)  =E= 0;
E6e(t,r)$(not r_h(r))..  STO_I(t,r)        =E= 0;


E9..                     COST              =E= sum((alltec,r),   INVE(alltec,r)                  * cost_fix(alltec))
                                             + sum((alltec,r),   (capa0(alltec,r)-DECO(alltec,r))* cost_qfix(alltec))    //* 2)
                                             + sum((t,tec_thm,r),GENE(t,tec_thm,r)               * cost_var(tec_thm))
                                             + sum((r,rr),       NTCinv(r,rr)*km(r,rr))/2        * cost_NTC;


MODEL EMMA /all/;




*===================================
* b) constraints
*===================================


LOAD.LO(t,r)                     = loa(t,r);
INVE.UP("lign",r)$(not r_lign(r))= 0;
DECO.FX(tec_res,r)               = 0;
FLOW.FX(t,r,rr)$(not km(r,rr))   = 0;
FLOW.FX(t,rr,r)$(not km(rr,r))   = 0;
NTCinv.UP(r,rr)                  = inf;          // used to be limited by: ntc(r,rr);





*================================================================================================================================================
* 9 SOLVE
*================================================================================================================================================


loop((s,y),


* Include scenario files

$include scen\h_%HORIZON%.gms
$include scen\y_%YDIM%.gms
$include scen\s_%SDIM%.gms



* Write solver option files

$onecho > Cplex.opt
lpmethod=1
ppriind=1
*advind=1
*tuning tuning-result.txt
$offecho
$echo lpmethod=2 >cplex.op2


* Options

EMMA.optfile=%OPTFILE%;           // choose the option file
EMMA.reslim=50000;                // limits solving time to 14h
EMMA.limrow=0;                    // reduces the size of the listing file
EMMA.limcol=0;                    // reduces the size of the listing file
EMMA.savepoint=2;                 // saves bases for every solve


* Load Starting Values

if((series=1 and ord(s)=1),                      // use benchmarke bases for first five runs in series 1
if(ord(y)=1,
$IF EXIST scen\start_%HORIZON%_%YDIM%_%HOURS%_1.gdx          execute_loadpoint "scen\start_%HORIZON%_%YDIM%_%HOURS%_1";
);
if(ord(y)=2,
$IF EXIST scen\start_%HORIZON%_%YDIM%_%HOURS%_2.gdx          execute_loadpoint "scen\start_%HORIZON%_%YDIM%_%HOURS%_2";
);
if(ord(y)=3,
$IF EXIST scen\start_%HORIZON%_%YDIM%_%HOURS%_3.gdx          execute_loadpoint "scen\start_%HORIZON%_%YDIM%_%HOURS%_3";
);
if(ord(y)=4,
$IF EXIST scen\start_%HORIZON%_%YDIM%_%HOURS%_4.gdx          execute_loadpoint "scen\start_%HORIZON%_%YDIM%_%HOURS%_4";
);
if(ord(y)=5,
$IF EXIST scen\start_%HORIZON%_%YDIM%_%HOURS%_5.gdx          execute_loadpoint "scen\start_%HORIZON%_%YDIM%_%HOURS%_5";
);
);

if((series=2 and (ord(s)=1) and ord(y)=1),       // use benchmark bases for first run in series 2
$IF EXIST scen\start_%HORIZON%_%YDIM%_%HOURS%_1.gdx        execute_loadpoint "scen\start_%HORIZON%_%YDIM%_%HOURS%_1.gdx";
);

if((series=3 and (ord(y)=1)),                    // use redistribution bases for first five runs in redistribution (series 3)
$IF EXIST scen\start_redistribution_%HOURS%.gdx            execute_loadpoint "scen\start_redistribution_%HOURS%";
);





SOLVE EMMA USING LP minimizing COST;





*================================================================================================================================================
* 10 REPORTING
*================================================================================================================================================




*===================================
* a) Outcome paremeters
*===================================


* By technology and region

o_capa(alltec,r)                    = capa0(alltec,r) + INVE.L(alltec,r) - DECO.L(alltec,r);
o_capa("CHP",r)                     = sum(tec_chp, CAPACHP.L(tec_chp,r));
o_ygene(tec,r)                      = sum(t,GENE.L(t,tec,r)) / th * sc;
o_ygene("pump",r)                   = sum(t,i_costs("pump","eff") * GENE.L(t,"pump",r) - STO_I.L(t,r)) / th;
o_ygene("CHP",r)                    = sum((tec_chp,t), CAPACHP.L(tec_chp,r) * profile(t,"CHP",r) * avail(t,tec_chp,r)) * sc / th;
o_ygene("RES",r)                    = sum(tec_res,o_ygene(tec_res,r));
o_cost(alltec,r)                    = (INVE.L(alltec,r)*cost_fix(alltec) + (capa0(alltec,r)-DECO.L(alltec,r))*cost_qfix(alltec)) * sc;      // *2
o_cost(tec_thm,r)                   = o_cost(tec_thm,r) + sum(t,GENE.L(t,tec_thm,r)*cost_var(tec_thm)) * sc;


* By region

o_bp(r)                             = sum(t,LOAD.M(t,r)) / card(t) * th;
o_dem(r)                            = sum(t,loa(t,r)) / th * sc;
o_gen(r)                            = sum(tec,o_ygene(tec,r));
o_CO2(r)                            = sum(tec,o_ygene(tec,r) * i_costs(tec,"co2int") / i_costs(tec,"eff"));
o_cur(r)                            = (sum(t,LOAD.L(t,r)) - sum(t,loa(t,r))) * sc / th;
o_ASp(r)$r_h(r)                     = sum(t,-E5b.M(t,r)) * sc;
o_ASp(r)$(not r_h(r))               = sum(t,-E5a.M(t,r)) * sc;



* By technology and region (again)

o_share(tec,r)                      = o_ygene(tec,r) / o_dem(r) * 100;
o_share("wind",r)                   = (o_ygene("wind",r) - o_cur(r)) / o_dem(r) * 100;
o_share(tec,"all")                  = sum(r,o_ygene(tec,r)) / sum(r,o_dem(r)) * 100;
o_share("wind",r)                   = (o_ygene("wind",r) - o_cur(r)) / o_dem(r) * 100;
o_share("wind","all")               = sum(r,(o_ygene("wind",r) - o_cur(r))) / sum(r,o_dem(r)) * 100;


* Revenues

o_revS(tec,r)                       = sum(t,GENE.L(t,tec,r) * LOAD.M(t,r)) * sc;
o_revS("pump",r)                    = sum(t,(i_costs("pump","eff") * GENE.L(t,"pump",r) - STO_I.L(t,r)) * LOAD.M(t,r)) * sc;
o_revH(alltec,r)                    = eps;
o_revH(tec_chp,r)                   = - CAPACHP.L(tec_chp,r) * E3a.M(r) * sc; //  sum(t,GENE.L(t,tec_chp,r) * (E3.M(t,tec_chp,r))) * sc;
o_revA(alltec,r)                    = eps;
o_revA(tec_thm,r)$(not r_h(r))      = sum(t,GENE.L(t,tec_thm,r) * (-E5a.M(t,r))) * sc;
o_revA(tec_thm,r)$r_h(r)            = sum(t,GENE.L(t,tec_thm,r) * (-E5b.M(t,r))) * sc;
o_revA(tec_chp,r)$(not r_h(r))      = sum(t,(GENE.L(t,tec_chp,r) - CAPACHP.L(tec_chp,r)*profile(t,"CHP",r)*avail(t,tec_chp,r)) * (-E5a.M(t,r))) * sc;
o_revA(tec_chp,r)$r_h(r)            = sum(t,(GENE.L(t,tec_chp,r) - CAPACHP.L(tec_chp,r)*profile(t,"CHP",r)*avail(t,tec_chp,r)) * (-E5b.M(t,r))) * sc;
o_revA("pump",r)                    = sum(t,(i_costs("pump","eff") * GENE.L(t,"pump",r) + STO_I.L(t,r)) * (-E5b.M(t,r))) * sc;
o_rev(alltec,r)                     = o_revS(alltec,r) + o_revH(alltec,r)+ o_revA(alltec,r);


* Values (o_val)

o_val(r,s,y,"Gen Costs")            = sum(alltec,o_cost(alltec,r)) / th                                                     +eps;
o_val(r,s,y,"Gen Costs w sunk")     = sum(alltec,o_cost(alltec,r)) / th + sum(alltec,capa0(alltec,r)*cost_inv(alltec))*sc/th+eps;
o_val(r,s,y,"IC cost")              = sum(rr, NTCinv.L(r,rr)            * km(r,rr)) / 2 * cost_NTC / th * sc                +eps;
o_val(r,s,y,"IC cost w sunk")       = sum(rr,(NTCinv.L(r,rr)+ntc(r,rr)) * km(r,rr)) / 2 * cost_NTC / th * sc                +eps;
o_val(r,s,y,"Gen Costs nRES")       = sum(tec_nre,o_cost(tec_nre,r)) / th                                                   +eps;
o_val(r,s,y,"Gen Costs nRES w sunk")=(sum(tec_nre,o_cost(tec_nre,r) + capa0(tec_nre,r)*cost_inv(tec_nre))) / th             +eps;
o_val(r,s,y,"Gen Costs RES")        = sum(tec_res,o_cost(tec_res,r)) / th                                                   +eps;
o_val(r,s,y,"Gen Costs RES w sunk") =(sum(tec_res,o_cost(tec_res,r) + capa0(tec_res,r)*cost_inv(tec_res))) / th             +eps;
o_val(r,s,y,"Gen Costs w/o CO2")    = (sum(alltec,o_cost(alltec,r)) - o_CO2(r) * CO2) / th                                  +eps;

o_val(r,s,y,"Spot Expenses")        = sum(t,LOAD.M(t,r)*loa(t,r)) * sc / th                                                 +eps;
o_val(r,s,y,"Spot Income")          = sum(alltec,o_revS(alltec,r)) / th                                                     +eps;
o_val(r,s,y,"AS Income")            = sum(alltec,o_revA(alltec,r)) / th                                                     +eps;
o_val(r,s,y,"Heat Income")          = sum(alltec,o_revH(alltec,r)) / th                                                     +eps;
o_val(r,s,y,"Spot Conv Income")     = sum(tec_nre,o_revS(tec_nre,r)) / th                                                   +eps;

o_val(r,s,y,"Sunk gen cost")        = sum(alltec,capa0(alltec,r)*cost_inv(alltec)) / th * sc                                +eps;
o_val(r,s,y,"Sunk conv gen cost")   = sum(tec_nre,capa0(tec_nre,r)*cost_inv(tec_nre)) / th * sc                             +eps;
o_val(r,s,y,"Sunk IC cost")         = sum(rr,ntc(r,rr) * km(r,rr)) / 2 * cost_NTC / th * sc                                 +eps;
o_val(r,s,y,"Conv Rent")            = sum(tec_nre,o_rev(tec_nre,r)-o_cost(tec_nre,r)) / th                                  +eps;
o_val(r,s,y,"Conv Rent - sunk cost")= o_val(r,s,y,"Conv Rent") - o_val(r,s,y,"Sunk conv gen cost")                          +eps;
o_val(r,s,y,"RES tax")              =-sum(tec_res,o_rev(tec_res,r)-o_cost(tec_res,r)) / th                                  +eps;
o_val(r,s,y,"CO2 revenue")          = o_CO2(r) * CO2 / th                                                                   +eps;

o_val(r,s,y,"Consumer Rent")        = 100 - o_val(r,s,y,"Spot Expenses") - o_val(r,s,y,"AS Income") - o_val(r,s,y,"Heat Income") - o_val(r,s,y,"IC cost");
o_val(r,s,y,"Producer Rent")        = o_val(r,s,y,"Conv Rent");
o_val(r,s,y,"State Revenue")        = o_val(r,s,y,"CO2 revenue") - o_val(r,s,y,"RES tax");
o_val(r,s,y,"Economic Welfare")     = o_val(r,s,y,"Consumer Rent") + o_val(r,s,y,"Producer Rent") + o_val(r,s,y,"State Revenue");





*===================================
* b) Overall System (r_s)
*===================================

r_s("TWh","Demand"                       ,s,y)   = sum(r,o_dem(r))                                                       +eps;
r_s("/MWh","Base Price"                 ,s,y)   = sum(r,o_dem(r)*o_bp(r)) / sum(r,o_dem(r))                             +eps;
r_s("/MWh","Load Price"                 ,s,y)   = sum((t,r), LOAD.M(t,r) * loa(t,r)) / sum((t,r),loa(t,r)) * th         +eps;

r_s("G/a","Total Sys Cost"              ,s,y)   = COST.L / th* sc                                                       +eps;
r_s("G/a","Total Sys Cost w sunk"       ,s,y)   =(COST.L + sum((alltec,r),capa0(alltec,r)*cost_inv(alltec)) + sum((r,rr),+ntc(r,rr)*km(r,rr))/2*cost_NTC) / th * sc +eps;

r_s("G/a",var_o_val                     ,s,y)   = sum(r,o_val(r,s,y,var_o_val))                                         +eps;
r_s("/MWh",var_val                      ,s,y)   = r_s("G/a",var_val,s,y) / sum(r,o_dem(r)) * th                        +eps;

r_s("RES","Wind share"                  ,s,y)   = o_share("wind","all")                                                 +eps;
r_s("RES","Solar share"                 ,s,y)   = o_share("sola","all")                                                 +eps;
r_s("RES","Wind MV"                     ,s,y)   = sum(r,o_revS("wind",r)) / sum(r,o_ygene("wind",r)) - cost_bal("wind") +eps;
r_s("RES","Solar MV"                    ,s,y)   = sum(r,o_revS("sola",r)) / sum(r,o_ygene("sola",r)) - cost_bal("wind") +eps;
r_s("RES","Wind VF"                     ,s,y)   = r_s("RES","Wind MV",s,y) / r_s("/MWh","Load Price",s,y) * 100       +eps;
r_s("RES","Solar VF"                    ,s,y)   = r_s("RES","Solar MV",s,y) / r_s("/MWh","Load Price",s,y) * 100      +eps;


r_s("/t","CO2 price"                    ,s,y)   = co2                                                                   +eps;
r_s("t","CO2 emissions"                  ,s,y)   = sum(r,o_CO2(r))                                                       +eps;

r_s("GW NTC","New IC capacity"           ,s,y)   = sum((r,rr),NTCinv.L(r,rr)) / 2                                        +eps;

r_s("TWh_RES","RES Curtailment"          ,s,y)   = sum(r,o_cur(r))                                                       +eps;
r_s("% RES","RES Curtailment"            ,s,y)   $sum(r,o_ygene("RES",r)>0.1)    = sum(r,o_cur(r)) / sum(r,o_ygene("RES",r)) * 100                                       +eps;
r_s("/MWh_RES","RES LCOE"               ,s,y)   $sum(r,o_ygene("RES",r)>0.1)    = sum((tec_res,r),o_cost(tec_res,r)) / sum(r,o_ygene("RES",r))                          +eps;
r_s("/MWh_RES","RES LCOE w curtail"     ,s,y)   $sum(r,o_ygene("RES",r)>0.1)    = sum((tec_res,r),o_cost(tec_res,r)) /(sum(r,o_ygene("RES",r)) - sum(r,o_cur(r)))       +eps;
test2$(ord(y)=1) =  COST.L / th* sc;
r_s("/MWh_RES","RES System costs"       ,s,y)   $sum(r,o_ygene("RES",r))        = ((COST.L / th * sc) - test2) / (sum(r,o_ygene("RES",r)) - sum(r,o_cur(r))) * th       +eps;
r_s("/MWh_RES","RES System LCOE"        ,s,y)   = r_s("/MWh_RES","RES LCOE w curtail",s,y) + r_s("/MWh_RES","RES System costs",s,y);

r_s("Inst Capa",alltec                   ,s,y)   = sum(r,o_capa(alltec,r))                                               +eps;
r_s("Inst Capa",alltec       ,"today","today")   = sum(r,i_capa0(alltec,r))                                              +eps;
r_s("Generation",alltec                  ,s,y)   = sum(r,o_ygene(alltec,r))                                              +eps;




*===================================
* b) Country-specific (r_r, r_r2, r_vf, r_capa, r_capa2, r_ms, r_gene)
*===================================


* Country Summary

r_r(r,"Demand","TWh",s,y)                        = o_dem(r)                                                              +eps;

r_r(r,"Base Price","/MWh",s,y)                  = o_bp(r)                                                               +eps;
r_r(r,"Peak Price","/MWh",s,y)                  = sum(t$tpeak(t,"p"), LOAD.M(t,r)) / sum(t$tpeak(t,"p"), 1) * th        +eps;
r_r(r,"Off-Peak Price","/MWh",s,y)              = sum(t$tpeak(t,"op"),LOAD.M(t,r)) / sum(t$tpeak(t,"op"),1) * th        +eps;
r_r(r,"Load Price","/MWh",s,y)                  = sum(t,LOAD.M(t,r) * loa(t,r)) / sum(t,loa(t,r)) * th                  +eps;
r_r(r,"Scarcity Price","/MWh",s,y)              = smax(t,LOAD.M(t,r)) * th                                              +eps;
r_r(r,"P-to-OP","1",s,y)                         = r_r(r,"Peak Price","/MWh",s,y) / r_r(r,"Off-Peak Price","/MWh",s,y) +eps;
r_r(r,"# zero prices","1",s,y)                   = sum(t$(LOAD.M(t,r)*th<1),1) * sc                                      +eps;
r_r(r,"# scarcity prices","1",s,y)               = sum(t$(LOAD.M(t,r)*th=r_r(r,"Scarcity Price","/MWh",s,y)),1) * sc    +eps;

r_r(r,"RES subsidy","G",s,y)                    = sum(tec_res,o_cost(tec_res,r) - o_rev(tec_res,r)) / th                +eps;
r_r(r,"RES subsidy","/MWh",s,y)$o_ygene("RES",r)= sum(tec_res,o_cost(tec_res,r) - o_rev(tec_res,r)) / o_ygene("RES",r)  +eps;
r_r(r,"RES share","%",s,y)                       = sum(tec_res,o_share(tec_res,r))                                       +eps;

r_r(r,"w cap","GW",s,y)                          = o_capa("wind",r)                                                      +eps;
r_r(r,"w share","%",s,y)                         = o_share("wind",r)                                                     +eps;
r_r(r,"w av pri","/MWh",s,y)$o_ygene("wind",r)  = o_revS("wind",r) / o_ygene("wind",r)                                  +eps;
r_r(r,"w bal costs","/MWh",s,y)                 = o_ASp(r) * as_res * th / i_FLH(r,"FLH") + cost_bal("wind")            +eps;
r_r(r,"w LCOE","/MWh",s,y)                      = cost_fix("wind") / sum(t,profile(t,"wind",r)) * th                    +eps;
r_r(r,"w value factor","1",s,y)$o_ygene("wind",r)= o_revS("wind",r) / o_ygene("wind",r) / o_bp(r)                        +eps;
r_r(r,"w Spot rev","/KW*a",s,y)$o_capa("wind",r)= o_revS("wind",r) / o_capa("wind",r)                                   +eps;
r_r(r,"w AS rev","/KW*a",s,y)$o_capa("wind",r)  = o_revA("wind",r) / o_capa("wind",r)                                   +eps;
r_r(r,"w gr m val","/KW*a",s,y)                 = (- INVE.M("wind",r) + cost_fix("wind")) * sc                          +eps;
r_r(r,"w capa cost","/KW*a",s,y)                = cost_fix("wind") * sc                                                 +eps;

r_r(r,"s cap","GW",s,y)                          = o_capa("sola",r)                                                      +eps;
r_r(r,"s share","%",s,y)                         = o_share("sola",r)                                                     +eps;
r_r(r,"s av pri","/MWh",s,y)$o_ygene("sola",r)  = o_revS("sola",r) / o_ygene("sola",r)                                  +eps;
r_r(r,"s bal costs","/MWh",s,y)                 = o_ASp(r) * as_res * th / sum(t,profile(t,"sola",r)) + cost_bal("wind")+eps;
r_r(r,"s LCOE","/MWh",s,y)                      = cost_fix("sola") / sum(t,profile(t,"sola",r)) * th                    +eps;
r_r(r,"s value factor","1",s,y)$o_ygene("sola",r)= o_revS("sola",r) / o_ygene("sola",r) / o_bp(r)                        +eps;
r_r(r,"s Spot rev","/KW*a",s,y)$o_capa("sola",r)= o_revS("sola",r) / o_capa("sola",r)                                   +eps;
r_r(r,"s AS rev","/KW*a",s,y)$o_capa("sola",r)  = o_revA("sola",r) / o_capa("sola",r)                                   +eps;
r_r(r,"s gr m val","/KW*a",s,y)                 = (- INVE.M("sola",r) + cost_fix("sola") ) * sc                         +eps;
r_r(r,"s capa cost","/KW*a",s,y)                = cost_fix("sola") * sc                                                 +eps;

r_r(r,"curtailment","TWh",s,y)                   = o_cur(r)                                                              +eps;
r_r(r,"CO2 price_r","/t",s,y)                   = co2                                                                   +eps;
r_r(r,"CHP MR share","%",s,y)                    = o_ygene("CHP",r) / r_r(r,"Demand","TWh",s,y) * 100                    +eps;
r_r(r,"exist exp cap","GW",s,y)                  = sum(rr,ntc(r,rr))                                                     +eps;
r_r(r,"new exp cap","GW",s,y)                    = sum(rr,NTCinv.L(r,rr))                                                +eps;
r_r(r,"net exports","TWh",s,y)                   = sum((t,rr),FLOW.L(t,r,rr)) / th * sc                                  +eps;
r_r(r,"IC util","%",s,y)$sum(rr,NTC(r,rr))       = ((sum((t,rr)$(FLOW.L(t,r,rr)>0),FLOW.L(t,r,rr)) / (sum(rr,NTC(r,rr)+NTCinv.L(r,rr))*card(t))) + (sum((t,rr)$(FLOW.L(t,rr,r)>0),FLOW.L(t,rr,r)) / (sum(rr,NTC(rr,r)+NTCinv.L(rr,r))*card(t)))) * 100 + eps;
r_r(r,"Pri AS MR","/KW*a",s,y)$r_h(r)           = o_ASp(r)                                                              +eps;
r_r(r,"CO2 emissions_r","Mt",s,y)                = o_CO2(r)                                                              +eps;


r_r2(r,var_o_val,s,y)                            = o_val(r,s,y,var_o_val) / o_dem(r) * th;



* Capacity and Generation

r_capa(r,alltec,s,y)                             = o_capa(alltec,r)                                                      +eps;
r_capa(r,"CHP must-run",s,y)                     = o_capa("CHP",r)                                                       +eps;
r_capa(r,"dispatchable",s,y)                     = sum(alltec,o_capa(alltec,r)) - sum(tec_res,o_capa(tec_res,r)) - o_capa("CHP",r) + eps;
r_capa(r,"intermittent",s,y)                     = sum(tec_res,o_capa(tec_res,r))                                        +eps;
r_capa(r,alltec,"today","today")                 = i_capa0(alltec,r)                                                     +eps;
r_capa("all",alltec,s,y)                         = sum(r,o_capa(alltec,r))                                               +eps;
r_capa("all",alltec,"today","today")             = sum(r,i_capa0(alltec,r))                                              +eps;

r_gene(r,"CHP must-run",s,y)                     = o_ygene("CHP",r)                                                      +eps;
r_gene(r,"dispatchable",s,y)                     = sum(alltec,o_ygene(alltec,r)) - sum(tec_res,o_ygene(tec_res,r)) - o_ygene("CHP",r) + eps;
r_gene(r,"intermittent",s,y)                     = sum(tec_res,o_ygene(tec_res,r))                                       +eps;
r_gene(r,alltec,s,y)                             = o_ygene(alltec,r)                                                     +eps;
r_gene("all",alltec,s,y)                         = sum(r,o_ygene(alltec,r))                                              +eps;

r_capa2(r,alltec,y,s)                            = r_capa(r,alltec,s,y);
r_capa2(r,"CHP must-run",y,s)                    = r_capa(r,"CHP must-run",s,y);
r_capa2(r,"dispatchable",y,s)                    = r_capa(r,"dispatchable",s,y);
r_capa2(r,"intermittent",y,s)                    = r_capa(r,"intermittent",s,y);
r_capa2("all",alltec,y,s)                        = r_capa("all",alltec,s,y);
r_capa2(r,alltec,"today","today")                = r_capa(r,alltec,"today","today");




* Value Factors

r_vf("1","market share","wind",y)                = o_share("wind","all")                                                 +eps;
r_vf("1","market share","solar",y)               = o_share("sola","all")                                                 +eps;
r_vf(r,s,"wind",y)                               = r_r(r,"w value factor","1",s,y);
r_vf("all",s,"wind",y)                           = sum(r,r_r(r,"w value factor","1",s,y)*o_ygene("wind",r)) / sum(r,o_ygene("wind",r));
r_vf(r,s,"solar",y)                              = r_r(r,"s value factor","1",s,y);
r_vf("all",s,"solar",y)                          = sum(r,r_r(r,"s value factor","1",s,y)*o_ygene("sola",r)) / sum(r,o_ygene("sola",r));

r_vf2("1","market share","wind",y)               = o_share("wind","all")                                                 +eps;
r_vf2("1","market share","solar",y)              = o_share("sola","all")                                                 +eps;
r_vf2(s,r,"wind",y)                              = r_vf(r,s,"wind",y);
r_vf2(s,"all","wind",y)                          = r_vf("all",s,"wind",y);
r_vf2(s,r,"solar",y)                             = r_vf(r,s,"solar",y);
r_vf2(s,"all","solar",y)                         = r_vf("all",s,"solar",y);




* Market Share

test3(tec_res)$(ord(y)=1) = cost_fix(tec_res);
r_ms("1","cost reduction","wind",y)              = (test3("wind") - cost_fix("wind"))    / test3("wind")         * 100   +eps;
r_ms("1","cost reduction","solar",y)             = (test3("sola") - cost_fix("sola"))    / test3("sola")         * 100   +eps;
r_ms(r,s,"wind",y)                               = o_share("wind",r)                                                     +eps;
r_ms("all",s,"wind",y)                           = o_share("wind","all")                                                 +eps;
r_ms(r,s,"solar",y)                              = o_share("sola",r)                                                     +eps;
r_ms("all",s,"solar",y)                          = o_share("sola","all")                                                 +eps;

r_ms2("1","cost reduction","wind",y)             = (test3("wind") - cost_fix("wind"))    / test3("wind")         * 100   +eps;
r_ms2("1","cost reduction","solar",y)            = (test3("sola") - cost_fix("sola"))    / test3("sola")         * 100   +eps;
r_ms2(s,r,"wind",y)                              = o_share("wind",r)                                                     +eps;
r_ms2(s,"all","wind",y)                          = o_share("wind","all")                                                 +eps;
r_ms2(s,r,"solar",y)                             = o_share("sola",r)                                                     +eps;
r_ms2(s,"all","solar",y)                         = o_share("sola","all")                                                 +eps;




*===================================
* c) Technology details (r_tec, r_sto, r_NTC)
*===================================



* Details By technology

r_tec(r,s,alltec,"capa0 (GW)"            ,y)     = capa0(alltec,r)                                                       +eps;
r_tec(r,s,alltec,"new cap (GW)"          ,y)     = INVE.L(alltec,r)                                                      +eps;
r_tec(r,s,alltec,"decom cap (GW)"        ,y)     = DECO.L(alltec,r)                                                      +eps;
r_tec(r,s,alltec,"net gen (TWh)"         ,y)     = o_ygene(alltec,r)                                                     +eps;
r_tec(r,s,alltec,"New Fix Cost (G)"     ,y)     = INVE.L(alltec,r) * cost_fix(alltec) * sc / th                         +eps;
r_tec(r,s,alltec,"Old qfix Cost (G)"    ,y)     = (capa0(alltec,r) - DECO.L(alltec,r)) * cost_qfix(alltec)  * sc / th   +eps;   //* 2
r_tec(r,s,tec,   "Var Cost (G)"         ,y)     = sum(t,GENE.L(t,tec,r)*cost_var(tec)) * sc / th                        +eps;
r_tec(r,s,alltec,"Revenue Spot (G)"     ,y)     = o_revS(alltec,r) / th                                                 +eps;
r_tec(r,s,alltec,"Revenue AS (G)"       ,y)     = o_revA(alltec,r) / th                                                 +eps;
r_tec(r,s,alltec,"Revenue Heat (G)"     ,y)     = o_revH(alltec,r) / th                                                 +eps;
r_tec(r,s,alltec,"SR Profit (G)"        ,y)     = (o_rev(alltec,r) - o_cost(alltec,r)) / th                             +eps;

r_tec(r,s,tec,"FLH (h)",y)$o_capa(tec,r)         = o_ygene(tec,r) / o_capa(tec,r) * th                                   +eps;
r_tec(r,s,tec,"spo mrg (/KW*a)",y)$o_capa(tec,r)= sum(t,GENE.L(t,tec,r)*(LOAD.M(t,r)-cost_var(tec))) / o_capa(tec,r)*sc +eps;
r_tec(r,s,tec,"Spread (/MWh)",y)$o_ygene(tec,r) = sum(t,GENE.L(t,tec,r)*(LOAD.M(t,r)-cost_var(tec))) / o_ygene(tec,r)*sc+eps;
r_tec(r,s,tec,"Av pri (/MWh)",y)$o_ygene(tec,r) = o_revS(tec,r) / o_ygene(tec,r)                                        +eps;
r_tec(r,s,tec,"val fact (1)",y)$o_ygene(tec,r)   = o_revS(tec,r) / o_ygene(tec,r) / o_bp(r)                              +eps;
r_tec(r,s,tec,"gross margin (/KW*a)",y)         = (- INVE.M(tec,r) + cost_fix(tec)) * sc                                +eps;   // IS THIS CORRECT?
r_tec(r,s,alltec,"fix cost new (/KW*a)",y)      = cost_fix(alltec) * sc                                                 +eps;   // TAKE INTO ACCOUNT AVAILABILITY?
r_tec(r,s,alltec,"qfix cost old (/KW*a)",y)     = cost_qfix(alltec)  * sc                                               +eps;   //* 2
r_tec(r,s,tec,"bidding price (/MWh)",y)         = cost_var(tec) * th                                                    +eps;

r_tec(r,s,"TOT",r_tec_var,y)                     = sum(alltec,    r_tec(r,s,alltec,r_tec_var,y))                         +eps;
r_tec("ALL",s,alltec,r_tec_var,y)                = sum(r,         r_tec(r,s,alltec,r_tec_var,y))                         +eps;
r_tec("ALL",s,"TOT",r_tec_var,y)                 = sum((r,alltec),r_tec(r,s,alltec,r_tec_var,y))                         +eps;


* Storage (r_sto)
* [this still needs to be finalized]
r_sto(r,"sto vol (GWh)",s,y)$o_capa("pump",r)    = o_capa("pump",r) * 8                                                  +eps;
r_sto(r,"pump cap (GW)",s,y)$o_capa("pump",r)    = o_capa("pump",r)                                                      +eps;
r_sto(r,"turb cap (GW)",s,y)$o_capa("pump",r)    = o_capa("pump",r) * i_costs("pump","eff")                              +eps;
r_sto(r,"FLH pum/tur (h)",s,y)$o_capa("pump",r)  = sum(t,GENE.L(t,"pump",r) + STO_I.L(t,r)) / o_capa("pump",r) * sc      +eps;
r_sto(r,"mar val PHS (/KW)",s,y)$o_capa("pump",r) = (sum(t,-E6a.M(t,r)-E6b.M(t,r))+8*sum(t,-E6c.M(t,r))) *sc            +eps;
r_sto(r,"mar val tur (/KW)",s,y)$o_capa("pump",r) =  sum(t,-E6a.M(t,r)-E6b.M(t,r)) * sc                                 +eps;
r_sto(r,"mar val vol (/8*KWh)",s,y)             = sum(t,-E6c.M(t,r)) * 8 * sc                                           +eps;
r_sto(r,"rev f AS (/KW*a)",s,y)$o_capa("pump",r)= sum(t,(GENE.L(t,"pump",r)*i_costs("pump","eff") + STO_I.L(t,r)) * (-E5b.M(t,r))) / o_capa("pump",r) * sc +eps;
r_sto(r,"rev f sp (/KW*a)",s,y)$o_capa("pump",r)= sum(t,(GENE.L(t,"pump",r)*i_costs("pump","eff") - STO_I.L(t,r)) *  LOAD.M(t,r) ) / o_capa("pump",r) * sc +eps;
r_sto(r,"cost (/KW*a)",s,y)$o_capa("pump",r)    = cost_fix("pump") * sc                                                 +eps;



* Interconnectors
* [maybe ideas and code can be taken from EnSys]

r_NTC("NTC old","-",r,"-",rr)                    = ntc(r,rr)                                                             +eps;
r_NTC("NTC new",y,r,s,rr)                        = NTCinv.L(r,rr)                                                        +eps;




*===================================
* d) Price setting and Merit-Order
*===================================


* Price-setting plant (r_set)

r_set(tec_thm,r,s,y)                             = sum(t$(LOAD.M(t,r)=cost_var(tec_thm)),1) /card(t)*100                 +eps;
r_set("zero",r,s,y)                              = sum(t$(LOAD.M(t,r)=0),                1) /card(t)*100                 +eps;
r_set("other",r,s,y)                             = 100 - sum(tec_thm,r_set(tec_thm,r,s,y)) - r_set("zero",r,s,y)         +eps;
r_set("some exports bind",r,s,y)                 = sum(t$(smin(rr,E4a.M(t,r,rr)) < 0),1)  /card(t)*100                   +eps;
r_set("all exports bind",r,s,y)                  = sum(t$(smax(rr,E4a.M(t,r,rr)) < eps),1)  /card(t)*100                 +eps;  // doesn't seem to work
r_set("some imports bind",r,s,y)                 = sum(t$(smin(rr,E4b.M(t,rr,r)) < 0),1)  /card(t)*100                   +eps;
r_set("all imports bind",r,s,y)                  = sum(t$(smax(rr,E4b.M(t,rr,r)) < eps),1)  /card(t)*100                 +eps;  // doesn't seem to work
r_set("must run bind",r,s,y)$(not r_h(r))        = sum(t$(E5a.M(t,r) ne 0),1)  /card(t)*100                              +eps;
r_set("must run bind",r,s,y)$(r_h(r))            = sum(t$(E5b.M(t,r) ne 0),1)  /card(t)*100                              +eps;
r_set("wind at zero",r,s,y)                      = sum(t$(LOAD.M(t,r)=0),GENE.L(t,"wind",r))/th*sc/o_ygene("wind",r)*100 +eps;

r_set2(tec_thm,r,y,s)                            = r_set(tec_thm,r,s,y);
r_set2("zero",r,y,s)                             = r_set("zero",r,s,y);


* Merit-order (r_mo)

r_mo(r,"CHP",s,y,"capa")                         = sum((t,tec_chp),CAPACHP.L(tec_chp,r) * profile(t,"CHP",r) * avail(t,tec_chp,r)) / card(t) + eps;
r_mo(r,tec,s,y,"capa")                           = o_capa(tec,r) * sum(t,avail(t,tec,r)) / card(t)                       +eps;
r_mo(r,tec_chp,s,y,"capa")                       = r_mo(r,tec_chp,s,y,"capa") - sum(t,CAPACHP.L(tec_chp,r) * profile(t,"CHP",r) * avail(t,tec_chp,r)) / card(t);
r_mo(r,tec,s,y,"var cost")                       = cost_var(tec) * th                                                    +eps;





*===================================
* e) Model stats
*===================================


r_mod(s,y,"modelstat")                           = EMMA.modelstat                                                        +eps;
r_mod(s,y,"seconds")                             = EMMA.resusd                                                           +eps;
r_mod(s,y,"hours")                               = EMMA.resusd / 3600                                                    +eps;
r_mod(s,y,"Iterations")                          = EMMA.iterusd                                                          +eps;
r_mod(s,y,"Hours modeled")                       = card(t)                                                               +eps;
r_mod(s,y,"Countries modeled")                   = card(r)                                                               +eps;





*===================================
* f) Error checking & development
*===================================


* Hourly (r_t)

if(ord(y)<3,
r_t(t,s,y,alltec,"GW")                           = GENE.L(t,alltec,"FRA")                                                +eps;
r_t(t,s,y,"pump","GW")                           = GENE.L(t,"pump","FRA") * i_costs("pump","eff")                        +eps;
r_t(t,s,y,"pump","GW")$(GENE.L(t,"pump","FRA")<little)  = - STO_I.L(t,"FRA")                                             +eps;
r_t(t,s,y,"CHP","GW")                            = sum(tec_chp,capaCHP.L(tec_chp,"FRA")*profile(t,"CHP","FRA")*avail(t,tec_chp,"FRA"))+eps;
r_t(t,s,y,"price","/MWh")                       = LOAD.M(t,"FRA") * th                                                  +eps;
r_t(t,s,y,"EEX","/MWh")                         = pri(t,"FRA")                                                          +eps;
r_t(t,s,y,"sto_i","GW")                          = STO_I.L(t,"FRA")                                                      +eps;
r_t(t,s,y,"load","GW")                           = loa(t,"FRA")                                                          +eps;
r_t(t,s,y,"must run","GW")                       = sum(tec_res,GENE.L(t,tec_res,"FRA")) + as("FRA") + r_t(t,s,y,"CHP","GW")+eps;
r_t(t,s,y,"res load","GW")                       = LOAD.L(t,"FRA") - r_t(t,s,y,"must run","GW")                          +eps;
r_t(t,s,y,"RL + ex cap","GW")                    = r_t(t,s,y,"res load","GW") + sum(rr,ntc("FRA",rr)+NTCinv.L("FRA",rr)) +eps;
r_t(t,s,y,"Net Exp","GW")                        = sum(rr,FLOW.L(t,"FRA",rr))                                            +eps;
);

r_t2(t,r,alltec,"GW")                            = GENE.L(t,alltec,r)                                                    +eps;
r_t2(t,r,"pump","GW")                            = GENE.L(t,"pump",r) * i_costs("pump","eff")                            +eps;
r_t2(t,r,"pump","GW")$(GENE.L(t,"pump",r)<little) = - STO_I.L(t,r)                                                       +eps;
r_t2(t,r,"CHP","GW")                             = sum(tec_chp,capaCHP.L(tec_chp,r)*profile(t,"CHP",r)*avail(t,tec_chp,r))+eps;
r_t2(t,r,"price","/MWh")                        = LOAD.M(t,r) * th                                                      +eps;
r_t2(t,r,"EEX","/MWh")                          = pri(t,r)                                                              +eps;
r_t2(t,r,"sto_i","GW")                           = STO_I.L(t,r)                                                          +eps;
r_t2(t,r,"load","GW")                            = loa(t,r)                                                              +eps;
r_t2(t,r,"must run","GW")                        = sum(tec_res,GENE.L(t,tec_res,r)) + as(r) + r_t2(t,r,"CHP","GW")       +eps;
r_t2(t,r,"res load","GW")                        = LOAD.L(t,r) - r_t2(t,r,"must run","GW")                               +eps;
r_t2(t,r,"RL + ex cap","GW")                     = r_t2(t,r,"res load","GW") + sum(rr,ntc(r,rr)+NTCinv.L(r,rr))          +eps;
r_t2(t,r,"Net Exp","GW")                         = sum(rr,FLOW.L(t,r,rr))                                                +eps;

r_t3(t,r,s,y)                                    = LOAD.M(t,r) * th                                                      +eps;

if(ord(s)=1,
r_t4(ti,y,alltec)                                = GENE.L(ti,alltec,"FRA")                                               +eps;
r_t4(ti,y,"Net Exp")                             = sum(rr,FLOW.L(ti,"FRA",rr))                                           +eps;
);



* Daily (r_d)

r_d(day,r,s,y,"price")                           = sum(t$tday(t,day), LOAD.M(t,r)) / 24 * th                             +eps;



* Calibration (r_cal)
r_cal("B-price",y,r,"his")                       = sum(t,pri(t,r)) / card(t)                                             +eps;
r_cal("B-price",y,r,"mod")                       = o_bp(r)                                                               +eps;
r_cal("P-price",y,r,"his")                       = sum(t$tpeak(t,"p"),   pri(t,r)) / sum(t$tpeak(t,"p"), 1)              +eps;
r_cal("P-price",y,r,"mod")                       = sum(t$tpeak(t,"p"),   LOAD.M(t,r)) / sum(t$tpeak(t,"p"), 1) * th      +eps;
r_cal("OP-price",y,r,"his")                      = sum(t$(tpeak(t,"op")),pri(t,r)) / sum(t$tpeak(t,"op"),1)              +eps;
r_cal("OP-price",y,r,"mod")                      = sum(t$(tpeak(t,"op")),LOAD.M(t,r)) / sum(t$tpeak(t,"op"),1) * th      +eps;
r_cal("P/OP ratio",y,r,"his")                    = r_cal("P-price",y,r,"his") / r_cal("OP-price",y,r,"his");
r_cal("P/OP ratio",y,r,"mod")                    = r_cal("P-price",y,r,"mod") / r_cal("OP-price",y,r,"mod");

r_cal2(r,y,rr,"BP dif","market")                 = eps;
r_cal2(r,y,rr,"BP dif","market")$km(r,rr)        = (sum(t,pri(t,r)) - sum(t,pri(t,rr))) / card(t)                        +eps;
r_cal2(r,y,rr,"BP dif","model")$km(r,rr)         = o_bp(r) - o_bp(rr)                                                    +eps;
r_cal2(r,y,rr,"HP dif","market")$km(r,rr)        = sum(t,abs(pri(t,r) - pri(t,rr))) / card(t)                            +eps;
r_cal2(r,y,rr,"HP dif","model")$km(r,rr)         = sum(t,abs(LOAD.M(t,r) - LOAD.M(t,rr))) / card(t) * th                 +eps;

r_cal3("price",r,y,"his",hourd)                  = sum(t$thourd(t,hourd),   pri(t,r)) / sum(t$thourd(t,hourd), 1)        +eps;
r_cal3("price",r,y,"mod",hourd)                  = sum(t$thourd(t,hourd),LOAD.M(t,r)) / sum(t$thourd(t,hourd), 1) * th   +eps;

r_cal4("price",r,y,"his",hourw)$(card(t)>265)    = sum(t$thourw(t,hourw),   pri(t,r)) / sum(t$thourw(t,hourw), 1)        +eps;
r_cal4("price",r,y,"mod",hourw)$(card(t)>265)    = sum(t$thourw(t,hourw),LOAD.M(t,r)) / sum(t$thourw(t,hourw), 1) * th   +eps;
r_cal4("load",r,y,"his",hourw)$(card(t)>265)     = sum(t$thourw(t,hourw),   loa(t,r)) / sum(t$thourw(t,hourw), 1)        +eps;
r_cal4("res load",r,y,"his",hourw)$(card(t)>265) = sum(t$thourw(t,hourw),   loa(t,r) - sum(tec_res,GENE.L(t,tec_res,r))) / sum(t$thourw(t,hourw), 1)        +eps;
r_cal4("CHP prof",r,y,"his",hourw)$(card(t)>265) = sum(t$thourw(t,hourw),   profile(t,"CHP",r))/sum(t$thourw(t,hourw), 1)+eps;




); // closes loop(s,y)



*===================================
* g) Reporting outside the loop
*===================================


r_cal("B-price","08-10",r,"his")                 = sum(y,r_cal("B-price",y,r,"his")) / 3                                 +eps;
r_cal("B-price","08-10",r,"mod")                 = sum(y,r_cal("B-price",y,r,"mod")) / 3                                 +eps;
r_cal("P-price","08-10",r,"his")                 = sum(y,r_cal("P-price",y,r,"his")) / 3                                 +eps;
r_cal("P-price","08-10",r,"mod")                 = sum(y,r_cal("P-price",y,r,"mod")) / 3                                 +eps;
r_cal("OP-price","08-10",r,"his")                = sum(y,r_cal("OP-price",y,r,"his")) / 3                                +eps;
r_cal("OP-price","08-10",r,"mod")                = sum(y,r_cal("OP-price",y,r,"mod")) / 3                                +eps;
r_cal("P/OP ratio","08-10",r,"his")              = sum(y,r_cal("P/OP ratio",y,r,"his")) / 3                              +eps;
r_cal("P/OP ratio","08-10",r,"mod")              = sum(y,r_cal("P/OP ratio",y,r,"mod")) / 3                              +eps;

r_cal3("price",r,"08-10","his",hourd)            = sum(y,r_cal3("price",r,y,"his",hourd)) / 3                            +eps;
r_cal3("price",r,"08-10","mod",hourd)            = sum(y,r_cal3("price",r,y,"mod",hourd)) / 3                            +eps;

r_cal4("price",r,"08-10","his",hourw)$(card(t)>265)      = sum(y,r_cal4("price",r,y,"his",hourw)) / 3                    +eps;
r_cal4("price",r,"08-10","mod",hourw)$(card(t)>265)      = sum(y,r_cal4("price",r,y,"mod",hourw)) / 3                    +eps;
r_cal4("load",r,"08-10","his",hourw)$(card(t)>265)       = sum(y,r_cal4("load",r,y,"his",hourw)) / 3                     +eps;

r_mod("tot","tot","seconds")                     = timeelapsed;
r_mod("tot","tot","hours")                       = timeelapsed / 3600;
r_mod("tot","av per solve","seconds")            = timeelapsed / card(s) / card(y);
r_mod("tot","av per solve","hours")              = timeelapsed / card(s) / card(y) / 3600;
r_mod("solving","tot","seconds")                 = sum((s,y),r_mod(s,y,"seconds"));
r_mod("non-solving","tot","seconds")             = timeelapsed - sum((s,y),r_mod(s,y,"seconds"));




*================================================================================================================================================
* 11 COUTPUT FILES
*================================================================================================================================================


execute_UNLOAD 'all.gdx';
execute_UNLOAD 'res.gdx'
r_s
r_r
r_r2
r_capa
r_capa2
r_gene
r_vf
r_vf2
r_ms
r_ms2
r_tec
r_sto
r_NTC
r_t
r_t2
r_t3
r_t4
r_d
r_set
r_set2
r_mo
r_cal
r_cal2
r_cal3
r_cal4
r_mod
;


$onecho > puts.txt
par=r_s          rng=sys!a1              cdim=2
par=r_r          rng=countries!a1        cdim=2
par=r_r2         rng=c2!a1               cdim=2
par=r_vf         rng=vf!a1               cdim=2
par=r_vf2        rng=vf!n1               cdim=2
par=r_capa       rng=capa!a1             cdim=2
par=r_capa2      rng=ca2!a1              cdim=2
par=r_ms         rng=ms!a1               cdim=2
par=r_ms2        rng=ms!n1               cdim=2
par=r_gene       rng=gene!a1             cdim=2
par=r_tec        rng=tec!a1              cdim=2
par=r_sto        rng=storage!a1          cdim=2
par=r_NTC        rng=NTC!a1              cdim=2
*par=r_t          rng=hourly!a1           cdim=4
*par=r_t2         rng=h2!a1               cdim=3
*par=r_t3         rng=h3!a2               cdim=3
*par=r_t4         rng=h4!a2               cdim=2
*par=r_d          rng=daily!a1            rdim=1
par=r_set        rng=p-setting!a1        rdim=1
par=r_set2       rng=p2!a1               rdim=1
par=r_mo         rng=merit-order!a20     rdim=2
*par=r_cal        rng=cal!a1              rdim=2
*par=r_cal2       rng=cal!a25             rdim=3
*par=r_cal3       rng=cal3!a1             rdim=4
*par=r_cal4       rng=cal4!a1             rdim=4
par=r_mod        rng=modelstats!a1
$offecho


execute "XLSTALK -c      %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%.xls"
execute "del             %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%.*"
execute "copy            input\res_template.xls  res.xls"
execute "ren             res.xls                 %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%.xls"
execute "ren             all.gdx                 %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%.gdx"
execute "GDXXRW.EXE      res.gdx   EpsOut=0    o=%PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%.xls @puts.txt"
*execute "XLSTALK -o                              %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%.xls"                                  // for testing
*execute "ren             res.xls                 %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%_op%OPTFILE%.xls"                     // for solver setting tests
*execute "ren             all.gdx                 %PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%_op%OPTFILE%.gdx"                     // for solver setting tests
*execute "GDXXRW.EXE      res.gdx   EpsOut=0    o=%PURPOSE%_%YDIM%_%HORIZON%_%HOURS%_%SDIM%_op%OPTFILE%.xls @puts.txt"           // for solver setting tests



if(series=1,
execute "del     scen\start_%HORIZON%_%YDIM%_%HOURS%_*.gdx"
execute "ren     EMMA_p1.gdx      start_%HORIZON%_%YDIM%_%HOURS%_1.gdx"
execute "ren     EMMA_p2.gdx      start_%HORIZON%_%YDIM%_%HOURS%_2.gdx"
execute "ren     EMMA_p3.gdx      start_%HORIZON%_%YDIM%_%HOURS%_3.gdx"
execute "ren     EMMA_p4.gdx      start_%HORIZON%_%YDIM%_%HOURS%_4.gdx"
execute "ren     EMMA_p5.gdx      start_%HORIZON%_%YDIM%_%HOURS%_5.gdx"
);

if(series=3,
execute "del     scen\VoS_redistribution_%HOURS%.gdx"
execute "ren     EMMA_p5.gdx      start_redistribution_%HOURS%.gdx"
);


execute "copy    start_*%HOURS%*.gdx     scen"
execute "del     start_*.gdx"
