



*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------
* This GAMS file is part of the Electricity Market Model EMMA, written by Lion Hirth.
* The model is published under the Creative Commons BY SA License (http://creativecommons.org/licenses/by-sa/3.0/).
* If you do not understand the license agreement, don't use the model.
* Please cite as Hirth, Lion (2013): "The Market Value of Variable Renewables", Energy Economics.
* Feedback, remarks, bug reportings, and suggestions are highly welcome: lion.hirth@vattenfall.com
*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------
*---------------------------------------------------------------------------------------------------------------------------








* --HOURS=100            full run takes 10min            only for error checking and routine testing
* --HOURS=4000           full run takes 30h              value factors are very close to 8760 run
* --HOURS=8760           full run takes four days        purpose?

* --PURPOSE=*            string, can be freely chosen, for identification

* --SERIES=1             to identify starting values: market value and market share runs (starting values respective benchmark)
* --SERIES=3             to identify starting values: redistribution runs (starting values (distributed-tailored) long-term runs)
* --SERIES=4             to identify starting values: other runs (no starting values)

* --YDIM=bench           five identical runs
* --YDIM=windshare       market share of wind is varied
* --YDIM=solarshare      market share of solar is varied
* --YDIM=CO2             CO2 price is varied
* --YDIM=REScosts        RES cost reduction is varied

*--SDIM=bench            no s SDIM                                       all
*--SDIM=Solar            solar share also increased                      only with windshare
*--SDIM=CO2              CO2 price varied                                only with windshare, solarshare, REScosts
*--SDIM=NoNuc            no nuclear (& CCS) allowed for investments      all
*--SDIM=Fuel             gas and coal price increased                    all
*--SDIM=NTC              NTC capacities varied                           all
*--SDIM=Storage          storage capacities varied                       all
*--SDIM=Flex             plant and system flexibility increased          all
*--SDIM=PriceCap         price cap reduced                               all
*--SDIM=WindProfiles     wind and solar profiles changed                 all
*--SDIM=Forerunner       all wind in Germany                             only with windshare
*--SDIM=Horizon          short, medium, long term benchmark runs         all
*--SDIM=Backtest         testing LTE vs. today                           ?
*--SDIM=SmallSteps       increasing wind share in smaller steps          only with windshare
*--SDIM=HighShares       increasing wind & solar share to higher levels  only with windshare
*--SDIM=HighReductions   increasing RES costs dramatically               only with REScosts
*--SDIM=CheapRES         RES costs reduced                               only with CO2

*--HORIZON=mid
*--HORIZON=short
*--HORIZON=long
*--HORIZON=none


$set HOURS      8760
$set PURPOSE    OptimalShare
$set HORIZON    long
$set YDIM       REScosts


$if not set HOURS        $set HOURS      100
$if not set PURPOSE      $set PURPOSE    WithoutPurpose
$if not set YDIM         $set YDIM       windshare
$if not set OPTFILE      $set OPTFILE    1
$if not set HORIZON      $set HORIZON    long




$ontext
* Runs that use benchmark starting values (series 1)
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=bench"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=windshare  --SDIM=Solar"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=CO2"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=NoNuc"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=Fuel"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=FuelElasticity"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=Invest"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=CostElasticity"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=NTC"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=Storage"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=Flex"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=Variability"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=PriceCap"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --YDIM=%YDIM%     --SDIM=WindProfiles"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=mid        --YDIM=%YDIM%     --SDIM=Horizon"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=mid        --YDIM=REScosts   --SDIM=Phase-Out"
$offtext


$ontext
* Runs that do not use benchmark starting values (series 2)
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=windshare  --SDIM=Backtest"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=windshare  --SDIM=SmallSteps"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=windshare  --SDIM=HighShares"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=REScosts   --SDIM=HighReductions"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=CO2        --SDIM=CheapRES"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=CO2        --SDIM=CheapRES2"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=CO2        --SDIM=HighRES"
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%  --HORIZON=%HORIZON%  --SERIES=2  --YDIM=CO2        --SDIM=50VRE"
$offtext


$ontext
* Redistribution run (series 3)
execute "gams    EMMA.gms  --HOURS=%HOURS%  --PURPOSE=%PURPOSE%  --OPTFILE=%OPTFILE%    --SERIES=3                        --SDIM=Redistribution"
$offtext




execute "mkdir                           %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%"
execute "mkdir                           %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%\inputs"
execute "mkdir                           %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%\GDX"
execute "copy    *%HOURS%*.xls           %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%"
execute "copy    *%HOURS%*.gdx           %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%\GDX"
execute "copy    *.gms                   %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%\inputs"
execute "copy    input\data*             %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%\inputs"
execute "copy    scen\*%HOURS%*.gdx      %PURPOSE%_%HOURS%_%HORIZON%_%date:~6,4%-%date:~3,2%-%date:~0,2%_%time:~1,1%%time:~3,2%\inputs"
execute "del     *%HOURS%*"








